//= require <dSS/GroupDialogs>

Ext.namespace('dSS');

dSS.GroupView = Ext.extend(Ext.DataView, {
    initComponent: function() {

        var groupTemplate = new Ext.XTemplate(
            '<tpl for=".">',
            '<div class="zone-wrap" id="group-{id}">',
            '<tpl if="css.length &gt; 0">',
                '<tpl if="(locktip.length &gt; 0)">',
                    '<table ext:qtip="{locktip}">',
                '</tpl>',
                '<tpl if="(locktip.length == 0)">',
                    '<table>',
                '</tpl>',
                '<tbody><tr>',
                    '<td><div class="{css} x-icon-combo-icon"></div></td>',
                    '<td>{name}</td>',
                '</tr></tbody></table>',
            '</tpl>',
            '<tpl if="(css.length == 0)">',
                '<span>{name}</span>',
            '</tpl>',
            '</div>',
            '</tpl>',
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            bubbleEvents: [ 'groupselection' ],
            store: dSS.data.Loader.getGroupStore(),
            tpl: groupTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: dSS._("No groups to display")
        });

        dSS.GroupView.superclass.initComponent.apply(this, arguments);

        dSS.data.Loader.getDeviceStore().on('load', function() {
                this.filterDevices();
            },
            this);

        this.on('selectionchange', function(dv, nodes) {
            if (nodes.length > 0) {
                var selectedGroups = this.getSelectedRecords();
                if (selectedGroups.length > 0) {
                    this.lastSelectedGroupID = selectedGroups[0].get('id');
                 }
            }

            var selection = this.getSelectedRecords();
            if (selection.length > 0) {
                var record = selection[0];
                var allowrm = dSS.util.isUserGroup(record.get('id'));
                this.refOwner.getTopToolbar().tbRemoveGroup.setDisabled(!allowrm);
                this.refOwner.getTopToolbar().tbEditGroup.setDisabled(!allowrm);

                if ((dSS.util.isUserGroup(record.get('id')) === true) &&
                    (record.get('locked') === true)) {
                    allowrm = false;
                }
                dSS.MessageBus.fireEvent('groupselection', record, allowrm);
            }
            this.filterDevices();
        },
        this);

        // do not allow to select the separator
        this.on('beforeselect', function(dv, node, selection) {
                if ((node.id == "group--1") || (node.id == "group--2")) {
                    return false;
                }

                return true;
            },
            this);

        dSS.MessageBus.on('resetgroupselection', function() {
                this.select(0);
            },
            this);

        dSS.MessageBus.on('structureloaded', function() {
            if (this.ownerCt.ownerCt) {
                if (!this.ownerCt.ownerCt.isVisible()) {
                    return;
                }
            }

            if (!this.ownerCt.isVisible()) {
                return;
            }

            if (!this.isVisible()) {
                return;
            }
            if (this.getStore().getCount() === 0) {
                this.getStore().clearFilter();
                this.filterDevices();
                return;
            }

            this.restoreSelection();
        },
        this);

        this.on('render', function() {
            var groupView = this;
            this.dropZone = new Ext.dd.DropZone(groupView.getEl(), {
                ddGroup: "groupDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(groupView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, source, e, data) {
                    var record = groupView.getRecord(target);
                    var grpSelection = groupView.getSelectedRecords();
                    var sourceGroup = 0;
                    var dropOK = true;

                    if (grpSelection.length > 0) {
                        sourceGroup = grpSelection[0].id;
                    }

                    // prevent self-drops
                    if (!dSS.util.isUserGroup(sourceGroup) &&
                        !dSS.util.isUserGroup(record.get('id'))) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // if source check passed above, prevent drops to all non
                    // user groups except special case "0" which surprisingly
                    // equals to "remove from group"
                    if ((record.get('id') > 0) &&
                        !dSS.util.isUserGroup(record.get('id'))) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // non present devices cannot be changed
                    Ext.each(data.selections, function(device) {
                        if (device.get('isPresent') === false) {
                            dropOK = false;
                            return false;
                        }
                    });
                    if (dropOK === false) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // #10051 prevent drops into locked clusters
                    if ((dSS.util.isUserGroup(record.get('id')) === true) &&
                        (record.get('locked') === true)) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // #10051 prevent drops of devices from locked clusters
                    if (grpSelection.length > 0) {
                       if ((dSS.util.isUserGroup(grpSelection[0].get('id')) === true) &&
                           (grpSelection[0].get('locked') === true)) {
                            return Ext.dd.DropZone.prototype.dropNotAllowed;
                        }
                    }

                    // special case: drop to group 0 means "remove"
                    if (record.get('id') === 0) {
                        return Ext.dd.DropZone.prototype.dropAllowed;
                    }

                    var alreadyInGroupCounter = 0;
                    Ext.each(data.selections, function(device) {
                        var selectedColor = dSS.util.getDeviceColor(device);
                        var pid = dSS.util.parseProductID(
                                                    device.get('productID'));
                        var numBinInputs = device.get('binaryInputCount');
                        var buttonId = device.get('buttonID');
                        var devType = pid[0];
                        if ((record.get('color') !== selectedColor) &&
                           (devType !== dSS.config.deviceTypeNum.AKM) &&
                           (!(devType === dSS.config.deviceTypeNum.UMR &&
                              numBinInputs > 0 &&
                              buttonId === dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL))) {
                            dropOK = false;
                            return false;
                        }

                        if (device.get('groups').indexOf(record.get('id'))
                                !== -1) {
                            alreadyInGroupCounter++;
                        }
                    });

                    if (dropOK === false) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // all selected devices are already in target group
                    if (alreadyInGroupCounter == data.selections.length) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = groupView.getRecord(target);
                    var grpSelection = groupView.getSelectedRecords();
                    var sourceGroup = 0;
                    if (grpSelection.length > 0) {
                        sourceGroup = grpSelection[0].id;
                    }

                    // prevent self-drops
                    if (!dSS.util.isUserGroup(sourceGroup) &&
                        !dSS.util.isUserGroup(record.get('id'))) {
                        return;
                    }

                    // #10051 prevent drops into locked clusters
                    if ((dSS.util.isUserGroup(record.get('id')) === true) &&
                        (record.get('locked') === true)) {
                        return;
                    }

                    // #10051 prevent drops of devices from locked clusters
                    if (grpSelection.length > 0) {
                       if ((dSS.util.isUserGroup(grpSelection[0].get('id')) === true) &&
                           (grpSelection[0].get('locked') === true)) {
                            return;
                        }
                    }

                    // if source check passed above, prevent drops to all non
                    // user groups except special case "0" which surprisingly
                    // equals to "remove from group"
                    if ((record.get('id') > 0) &&
                        !dSS.util.isUserGroup(record.get('id'))) {
                        return;
                    }

                    var groupColor = record.get('color');
                    var rmRecords = [];

                    Ext.each(data.selections, function(device) {
                        // skip non present devices
                        if (device.get('isPresent') === false) {
                            return true;
                        }

                        // selection takes care of only allowing same color
                        // devices, so if one does not match we can expect
                        // the whole selection to be bogus and stop iteration
                        var col = dSS.util.getDeviceColor(device);
                        var pid = dSS.util.parseProductID(
                                                    device.get('productID'));
                        var numBinInputs = device.get('binaryInputCount');
                        var buttonId = device.get('buttonID');
                        var devType = pid[0];
                        if (((col !== groupColor) &&
                             (devType !== dSS.config.deviceTypeNum.AKM)) &&
                             (!(devType === dSS.config.deviceTypeNum.UMR &&
                                numBinInputs > 0 &&
                                buttonId === dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL)) &&
                             (record.get('id') !== 0)) {
                            return false;
                        }

                        // group 0 - remove mode
                        if (record.get('id') === 0) {
                            rmRecords.push(device);
                            return true;
                        }

                        // skip devices that are already in the group
                        if (device.get('groups').indexOf(record.get('id'))
                                !== -1) {
                            return true;
                        }

                        Ext.Ajax.request({
                            url: '/json/structure/clusterAddDevice',
                            disableCaching: true,
                            method: "GET",
                            scope: groupView,
                            params: {
                                dsuid: device.get('dsuid'),
                                clusterID: record.get('id')
                            },
                            success: function(result, request) {
                                try {
                                    var json = Ext.util.JSON.decode(result.responseText);
                                    if (json.ok) {
                                        var store = dSS.data.Loader.getDeviceStore();
                                        if (json.result.action === "update") {
                                            dSS.util.processDeviceActionResponse(json.result);
                                            dSS.MessageBus.fireEvent(
                                                'statusmessage',
                                                dSS.gt.strargs(
                                                    dSS._("Device %1 was added"+
                                                        " to group %2"),
                                                    [
                                                        '&quot;' +
                                                        dSS.util.getName(
                                                            device.get('name'))+
                                                        '&quot; (' + device.get('DisplayID') +
                                                        ')',
                                                        '&quot;' +
                                                        record.get('name') +
                                                        '&quot;'
                                                    ]
                                                )
                                            );
                                        }
                                    } else {
                                        Ext.Msg.alert(dSS._("Error"),
                                            dSS._("Device") +
                                            ' ' + device.get('DisplayID') +
                                            ' ' + dSS._("could not be added " +
                                                "to group") + ' ' +
                                            record.get('name') +
                                            ': (' + json.message + ')');
                                    }
                                } catch(err) {
                                    Ext.Msg.alert(
                                        dSS._("Error"), dSS._("Device") + ' ' +
                                        device.get('DisplayID') + ' ' +
                                        dSS._("could not be added to group") +
                                        ' ' + record.get('name') +
                                        ': (' + err + ')');
                                    dSS.MessageBus.fireEvent('reloadapartment');
                                }
                            },
                            failure: function(result, request) {
                                Ext.Msg.alert(
                                    dSS._("Error"), dSS._("Device") +  ' ' +
                                    device.get('DisplayID') + ' ' +
                                    dSS._("could not be added to group") +
                                    ' ' + record.get('name'));
                                dSS.MessageBus.fireEvent('reloadapartment');
                            }
                        });
                    });

                    // this is actually a remove group action, no adding
                    // took place
                    if (rmRecords.length > 0) {
                        var grpSelection = groupView.getSelectedRecords();
                        if (grpSelection.length > 0) {
                            groupView.findParentByType('dssgroupbrowser').devicePanel.removePresentFromGroup(rmRecords, grpSelection[0].id);
                        }
                    }

                    return true;
                }
            });
        },
        this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            this.select(index);
            var sel = this.getSelectedRecords();
            var record = sel[0];

            var menuItem = new Ext.menu.Item({
                text: dSS._("Create new user group"),
                iconCls: 'addUserGroupIcon',
                handler: this.createGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            var modify = true;
            if (!dSS.util.isUserGroup(record.get('id'))) {
                modify = false;
            }
            menuItem = new Ext.menu.Item({
                text: dSS._("Edit user group"),
                iconCls: 'editUserGroupIcon',
                disabled: !modify,
                handler: this.editGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Remove user group"),
                iconCls: 'removeUserGroupIcon',
                disabled: !modify,
                handler: this.removeGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);


            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    restoreSelection: function() {
        // no action if there is a valid selection
        var selectedGroups = this.getSelectedRecords();
        if (selectedGroups.length > 0) {
            this.filterDevices();
            return;
        }

        if (this.lastSelectedGroupID !== undefined) {
            var rec = this.getStore().getById(this.lastSelectedGroupID);
            if (rec !== undefined) {
                this.select(rec);
            } else {
                this.select(this.store.getById(0));
            }
        } else {
            this.select(this.store.getById(0));
        }
    },


    filterDevices: function() {
        var selectedGroup;
        var selectedGroups = this.getSelectedRecords();
        if (selectedGroups.length > 0) {
            selectedGroup = selectedGroups[0]; // only single select is enabled
        }

        dSS.MessageBus.fireEvent('filter_devices_group', selectedGroup);
    },

    removeGroup: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No group selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var deviceStore = dSS.data.Loader.getDeviceStore();

        if (deviceStore.getRange().length > 0) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Can not delete a non empty group, please " +
                                "remove all devices from the group first."));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeCluster',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                clusterID: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                          dSS.gt.strargs(dSS._('Group "%1" was removed'),
                                  [record.get('name')]));
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove group") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove group") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove group"));
            }
        });
    },
    editGroup: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No group selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            title: dSS._("Edit user group"),
            rowSel: this,
            jumpMode: false,
            dialogType: dSS.config.dialogTypes.EDIT_GROUP_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el,
            width: 310
        });

        win.show();
    },
    createGroup: function() {
        var ugCount = 0;
        this.getStore().each(function(record) {
            if (dSS.util.isUserGroup(record.get('id'))) {
              ugCount++;
            }
            if (ugCount >= dSS.config.UserGroupMaxCount) {
                return false;
            }
            return true;
        });

        if (ugCount >= dSS.config.UserGroupMaxCount) {
          Ext.Msg.alert(dSS._("Warning"),
                        dSS._("Maximum number of user groups reached"));
          return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: this,
            title: dSS._("Create new user group"),
            dialogType: dSS.config.dialogTypes.CREATE_GROUP_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el,
            width: 300
        });

        win.show();
    }
});

Ext.reg('dssgroupview', dSS.GroupView);

