Ext.namespace('dSS', 'dSS.util');

dSS.util.UpdateRowSpan = function(scope, onlyfirst) {
    var columns = scope.getColumnModel().config;
    var view = scope.getView();
    var store = scope.getStore();
    var rowCount = store.getCount();

    var column = columns[0];
    var dataIndex = column.dataIndex;

    var column2 = columns[1];
    var dataIndex2 = column2.dataIndex;

    var spanCell = null;
    var spanCount = null;
    var spanValue = null;

    var spanCell2 = null;
    var spanCount2 = null;
    var spanValue2 = null;

    for (var row = 0; row < rowCount; ++row) {
        var cell = view.getCell(row, 0);
        var cell2 = view.getCell(row, 1);
        var record = store.getAt(row);
        var value = record.get(dataIndex);
        var value2 = record.get(dataIndex2);

        if (spanValue != value) {
            if (spanCell !== null) {
                spanCell.rowSpan = spanCount;
            }

            var c = Ext.fly(cell);
            c.setStyle('display', '');

            spanCell = cell;
            spanCount = 1;
            spanValue = value;
        } else {
            spanCount++;
            var c = Ext.fly(cell);
            c.update('');
            var d = c.findParent('div', 5, true);
            if (d) {
                d.addClass('grid-cell-no-borders');
            }
        }

        if (!onlyfirst) {
            if (spanValue2 != value2) {
                if (spanCell2 !== null) {
                    spanCell2.rowSpan = spanCount2;
                }

                var c = Ext.fly(cell2);
                c.setStyle('display', '');
                if (spanCount > 1) {
                    c.addClass('grid-top-border');
                    var cell3 = view.getCell(row, 2);
                    Ext.fly(cell3).addClass('grid-top-border');

                    // make sure all sensor rows have borders
                    var cell4 = view.getCell(row, 3);
                    var el = Ext.fly(cell4);
                    if (el) {
                        el.addClass('grid-top-border');
                    }
                }

                spanCell2 = cell2;
                spanCount2 = 1;
                spanValue2 = value2;
            } else {
                spanCount2++;
                var c = Ext.fly(cell2);
                c.update('');
            }
        }
    }

    if (spanCell !== null) {
        spanCell.rowSpan = spanCount;
    }

    if (!onlyfirst) {
        if (spanCell2 !== null) {
            spanCell2.rowSpan = spanCount2;
        }
    }
}

dSS.SystemStateTable = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        var store = new Ext.data.ArrayStore({
            autoDestroy: true,
            hasMultiSort: true,
            multiSortInfo:
            {
                direction: 'ASC',
                sorters:
                [
                    { field: 'sort', direction: 'ASC' },
                    { field: 'typesummary', direction: 'ASC' },
                    { field: 'type', direction: 'ASC' }
                ]
            },
            fields: [ 'sort', 'typesummary', 'type', 'state', 'sensors' ]
        });

        Ext.apply(this, {
            title: dSS._("System State"),
            store: store,
            viewConfig:
            {
                hideSortIcons: true,
                stripeRows: false,
                rowOverCls: ''
            },
            cls: 'grid-no-borders',
            columnLines: true,
            enableColumnHide: false,
            enableColumnMove: false,
            minColumnWidth: 80,
            enableHdMenu: false,
            columns:
            [
                {
                    header: dSS._("Type"),
                    tooltip: dSS._("Type"),
                    dataIndex: 'typesummary',
                    sortable: false,
                    width: 200
                },
                {
                    dataIndex: 'type',
                    sortable: false,
                    width: 160
                },
                {
                    header: dSS._("State"),
                    tooltip: dSS._("State"),
                    sortable: false,
                    dataIndex: 'state',
                    width: 180
                },
                {
                    header: dSS._("Sensor values"),
                    tooltip: dSS._("Sensor values"),
                    sortable: false,
                    dataIndex: 'sensors',
                    width: 120
                }
            ]
        });

        dSS.SystemStateTable.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
            this.getSelectionModel().clearSelections(true);
            this.getSelectionModel().lock();
        }, this);

        this.getView().on('refresh', this.updateRowSpan, this);
    },
    updateRowSpan: function() {
        dSS.util.UpdateRowSpan(this, false);
    }
});
Ext.reg ('dsssysstatetable', dSS.SystemStateTable);

dSS.ZoneStateTable = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        var store = new Ext.data.JsonStore({
            autoDestroy: true,
            sortInfo: { field: 'zone', direction: 'ASC' },
            root: 'data',
            fields:
            [
                'zone',
                dSS.config.groupNames[dSS.config.groups.LIGHT],
                dSS.config.groupNames[dSS.config.groups.SHADOW],
                dSS.config.groupNames[dSS.config.groups.HEATING],
                dSS.config.groupNames[dSS.config.groups.AUDIO],
                dSS.config.groupNames[dSS.config.groups.VIDEO],
                'sensors',
                'tempcontrol',
                dSS.config.groupNames[dSS.config.groups.VENTILATION],
                dSS.config.groupNames[dSS.config.groups.RECIRCULATION]
            ]
        });

        Ext.apply(this, {
            title: dSS._("Room State"),
            stripeRows: false,
            columnLines: true,
            viewConfig:
            {
                hideSortIcons: true,
                stripeRows: false,
                rowOverCls: ''
            },
            cls: 'grid-no-borders',
            enableColumnHide: false,
            enableColumnMove: false,
            enableHdMenu: false,
            store: store,
            columns:
            [
                {
                    header: dSS._("Room"),
                    tooltip: dSS._("Room"),
                    dataIndex: 'zone',
                    sortable: false
                },
                {
                    header: dSS._("Light"),
                    tooltip: dSS._("Light"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.LIGHT],
                    sortable: false
                },
                {
                    header: dSS._("Shade"),
                    tooltip: dSS._("Shade"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.SHADOW],
                    sortable: false
                },
                {
                    header: dSS._("Heating"),
                    tooltip: dSS._("Heating"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.HEATING],
                    sortable: false
                },
                {
                    header: dSS._("Audio"),
                    tooltip: dSS._("Audio"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.AUDIO],
                    sortable: false
                },
                {
                    header: dSS._("Video"),
                    tooltip: dSS._("Video"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.VIDEO],
                    sortable: false
                },
                {
                    header: dSS._("Temperature control"),
                    tooltip: dSS._("Temperature control"),
                    dataIndex: 'tempcontrol',
                    sortable: false
                },
                {
                    header: dSS._("Sensor values"),
                    tooltip: dSS._("Sensor values"),
                    dataIndex: 'sensors',
                    sortable: false
                },
                {
                    header: dSS._("Room ventilation"),
                    tooltip: dSS._("Room ventilation"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.VENTILATION],
                    sortable: false
                },
                {
                    header: dSS._("Recirculation"),
                    tooltip: dSS._("Recirculation"),
                    dataIndex: dSS.config.groupNames[dSS.config.groups.RECIRCULATION],
                    sortable: false
                }
            ]
        });

        dSS.ZoneStateTable.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
            this.getSelectionModel().clearSelections(true);
            this.getSelectionModel().lock();
        }, this);

        this.getView().on('refresh', this.updateRowSpan, this);
    },
    updateRowSpan: function() {
        dSS.util.UpdateRowSpan(this, true);
    }
});
Ext.reg ('dsszonestatetable', dSS.ZoneStateTable);

dSS.GroupStateTable = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {
        var store = new Ext.data.ArrayStore({
            fields: [ 'group', 'color', 'state', 'conflock', 'oplock' ]
        });

        Ext.apply(this, {
            title: dSS._("Group State"),
            viewConfig:
            {
                hideSortIcons: true,
                stripeRows: false,
                rowOverCls: ''
            },
            store: store,
            columns:
            [
                {
                    header: dSS._("Group"),
                    tooltip: dSS._("Group"),
                    dataIndex: 'group',
                    sortable: false
                },
                {
                    header: dSS._("Color group"),
                    tooltip: dSS._("Color group"),
                    dataIndex: 'color',
                    sortable: false
                },
                {
                    header: dSS._("State"),
                    tooltip: dSS._("State"),
                    dataIndex: 'state',
                    sortable: false
                },
                {
                    header: dSS._("Configuration"),
                    tooltip: dSS._("Configuration"),
                    dataIndex: 'conflock',
                    sortable: false
                },
                {
                    header: dSS._("Operation lock"),
                    tooltip: dSS._("Operation lock"),
                    datIndex: 'oplock',
                    sortable: false
                }
            ]
        });

        dSS.GroupStateTable.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
            this.getSelectionModel().clearSelections(true);
            this.getSelectionModel().lock();
        }, this);
    }
});
Ext.reg ('dssgroupstatetable', dSS.GroupStateTable);


dSS.SystemStatesPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        if (config && config.sizeObject) {
            this.sizeObject = config.sizeObject;
        }

        this.heightConfigured = false;

        Ext.apply(this, {
            title: dSS._("System states"),
            layout: 'border',
            items:
            [
                {
                    xtype: 'dsssysstatetable',
                    ref: 'systemstates',
                    split: true,
                    region: 'north'
                },
                {
                    xtype: 'dsszonestatetable',
                    ref: 'zonestates',
                    split: true,
                    region: 'center'
                },
                {
                    xtype: 'dssgroupstatetable',
                    ref: 'groupstates',
                    split: true,
                    region: 'south'
                }
            ]
        });

        dSS.SystemStatesPanel.superclass.constructor.apply(this, arguments);

        this.on('activate', function() {
            this.maskEl.mask(dSS._("Loading") + "...", 'x-mask-loading');
            this.requestData()
        }, this)


        dSS.MessageBus.on('refreshview', function() {
            if (this.isVisible()) {
                dSS.MessageBus.on('structureloaded',
                                  this.requestData.createDelegate(this),
                                  this, { single: true });
                this.maskEl.mask(dSS._("Loading") + "...", 'x-mask-loading');
                dSS.MessageBus.fireEvent('reloadapartment');
            }
        }, this);

        this.on('afterrender', function() {
            this.maskEl = this.findParentByType(Ext.Viewport).el;

            if (!this.heightConfigured) {
                if (!this.sizeObject) {
                    this.sizeObject = this.ownerCt;
                }

                if (this.sizeObject) {
                    var h = this.sizeObject.getInnerHeight() / 3;
                    this.systemstates.setHeight(h);
                    this.zonestates.setHeight(h);
                    this.groupstates.setHeight(h);
                    this.heightConfigured = true;
                }
            }
        }, this);
    },

    getHeatingControllerMode: function(system) {
        Ext.Ajax.request(
        {
            url: '/json/property/getString',
            method: 'GET',
            scope: this,
            params:
            {
                path: "/usr/addon-states/heating-controller/apartment_controller/state"
            },
            success: function(result) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (json.ok) {
                       try {
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("HEATING"),
                                dSS._("Heating"),
                                dSS._("Heating controller status"),
                                dSS._(json.result.value)
                            ]);
                        } catch (err) {}
                    }
                    this.getUserDefinedStates(system);
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not load state information from dSS") +
                              ": " + err);
                    this.getUserDefinedStates(system);
                }
            },
            failure: function(result) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not load state information from dSS"));
                this.getUserDefinedStates(system);
            }
        });
    },

    getUserDefinedStates: function(system) {
        Ext.Ajax.request(
        {
            url: '/json/property/query',
            method: 'GET',
            scope: this,
            params:
            {
                query: "/usr/addon-states/system-addon-user-defined-states/*(displayName,state,setName,resetName)"
            },
            success: function(result) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((json.ok) &&
                        (json.result["system-addon-user-defined-states"] !== undefined)) {
                        var states = json.result["system-addon-user-defined-states"];
                        var i;
                        for (i = 0; i < states.length; i++) {
                            var state = states[i];
                            var value = state.setName;
                            if (state.state == "inactive") {
                                value = state.resetName;
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("USER_DEFINED_STATES"),
                                dSS._("User defined states"),
                                state.displayName,
                                value
                            ]);
                        }
                    }
                    this.systemstates.getStore().loadData(system);
                    this.maskEl.unmask();
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not load state information from dSS") +
                              ": " + err);
                    this.systemstates.getStore().loadData(system);
                    this.maskEl.unmask();
                }
            },
            failure: function(result) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not load state information from dSS"));
                this.systemstates.getStore().loadData(system);
                this.maskEl.unmask();
            }
        });
    },

    requestGroupData: function(oplocks, system) {
        Ext.Ajax.request(
        {
            url: '/json/property/query',
            method: 'GET',
            scope: this,
            params:
            {
                query: "/apartment/zones/zone0/groups/*(group,lastCalledScene,lastCalledBasicScene,name,ConfigurationLocked,color)"
            },
            success: function(result) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not load state information from dSS") +
                            ': ' + json.message);
                        this.maskEl.unmask();
                        return;
                    }
                    var groupdata = [];
                    var groupStore = dSS.data.Loader.getGroupStore();

                    var i;
                    for (i = 0; i < json.result.groups.length; i++) {
                        var group = json.result.groups[i];

                        if (group.group === 64) {
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("APT_VENTILATION"),
                                dSS._("Apartment ventilation"),
                                "",
                                dSS._(dSS.config.sceneNamesAV[group.lastCalledBasicScene])
                            ]);
                            continue;
                        }

                        if ((group.group < 16) || (group.group > 47) ||
                            (group.color === 0)) {
                            continue;
                        }

                        var name = "";
                        var grouprec = groupStore.getById(group.group);
                        if (grouprec) {
                            name = grouprec.get('name');
                        }

                        if (name.length === 0) {
                            name = dSS._("Group") + " #" + group.group;
                        }

                        var lock = "not locked";
                        if (group.ConfigurationLocked === true) {
                            lock = "locked";
                        }

                        var oplock = "not locked";
                        if (oplocks[group.group] === true) {
                            oplock = "locked";
                        }
                        groupdata.push(
                        [
                            name,
                            dSS._(dSS.config.groupNames[group.color]),
                            dSS._(dSS.util.SceneToString(group.lastCalledScene)),
                            dSS._(lock),
                            dSS._(oplock)
                        ]);
                    }

                    this.groupstates.getStore().loadData(groupdata);
                    this.getHeatingControllerMode(system);
                    this.maskEl.unmask();
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not load state information from dSS") +
                              ": " + err);
                    this.maskEl.unmask();
                }
            },
            failure: function(result) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not load state information from dSS"));
                this.maskEl.unmask();
            }
        });
    },
    requestData: function() {
        this.maskEl.mask(dSS._("Loading") + "...", 'x-mask-loading');
        var initZone = function() {
            var zone = {};
            zone.zone = "";
            zone[dSS.config.groupNames[dSS.config.groups.LIGHT]] = "";
            zone[dSS.config.groupNames[dSS.config.groups.SHADOW]] = "";
            zone[dSS.config.groupNames[dSS.config.groups.HEATING]] = "";
            zone[dSS.config.groupNames[dSS.config.groups.AUDIO]] = "";
            zone[dSS.config.groupNames[dSS.config.groups.VIDEO]] = "";
            zone.sensors = "";
            zone.tempcontrol = "";
            zone[dSS.config.groupNames[dSS.config.groups.VENTILATION]] = "";
            zone[dSS.config.groupNames[dSS.config.groups.RECIRCULATION]] = "";
            return zone;
        };

        var oplocks = {};

        var zoneStore = dSS.data.Loader.getZoneStore();
        var groupStore = dSS.data.Loader.getGroupStore();

        var getSensorsForApartment = function(sensors) {
            var ret = [];
            if (sensors === undefined) {
                return ret;
            }

            console.log("Sensors: ", sensors);
            var s;
            for (s = 0; s < sensors.length; s++) {
                var sensor = sensors[s];
                var unit = dSS._(dSS.config.SensorTypeUnits[sensor.type]);
                var type = dSS._(dSS.config.SensorTypeStrings[sensor.type]);
                if (sensor.type === dSS.config.SensorTypes.ROOM_TEMP_SETPOINT) {
                    type = dSS._("Set temperature");
                } else if (sensor.type === dSS.config.SensorTypes.ROOM_TEMP_CONTROL_VAR) {
                    type = dSS._("Control value")
                }
                var value = String(Math.round(sensor.value * 10) / 10);
                var item = {
                    type: type,
                    value: value + " " + unit
                };
                ret.push(item);
            }

            ret.sort();
            return ret;
        };

        var getSensors = function(sensors, config) {
            var addType = config.addType;
            var ignoreTempCtl = config.ignoreTempCtl;
            var ignoreSetpoint = config.ignoreSetpoint;
            var onlySetpoint = config.onlySetpoint;

            var ret = "";
            if (sensors === undefined) {
                return ret;
            }
            var arr = [];
            var s;
            for (s = 0; s < sensors.length; s++) {
                var sensor = sensors[s];
                if (ignoreTempCtl === true) {
                    if (sensor.type == dSS.config.SensorTypes.ROOM_TEMP_CONTROL_VAR) {
                        continue;
                    }
                }

                if (ignoreSetpoint === true) {
                    if (sensor.type == dSS.config.SensorTypes.ROOM_TEMP_SETPOINT) {
                        continue;
                    }
                }

                if ((ignoreSetpoint !== true) && (onlySetpoint === true)) {
                    if (sensor.type != dSS.config.SensorTypes.ROOM_TEMP_SETPOINT) {
                        continue;
                    }
                }

                if (!dSS.util.isVisibleSensorType(sensor.type, true) &&
                    ((sensor.type != dSS.config.SensorTypes.ROOM_TEMP_SETPOINT)) &&
                     (sensor.type != dSS.config.SensorTypes.ROOM_TEMP_CONTROL_VAR)) {
                    continue;
                }

                var unit = dSS._(dSS.config.SensorTypeUnits[sensor.type]);
                var type = dSS._(dSS.config.SensorTypeStrings[sensor.type]);
                if (sensor.type === dSS.config.SensorTypes.ROOM_TEMP_SETPOINT) {
                    type = dSS._("Set temperature");
                } else if (sensor.type === dSS.config.SensorTypes.ROOM_TEMP_CONTROL_VAR) {
                    type = dSS._("Control value")
                }

                if (ret.length > 0) {
                    arr.push(ret);
                    ret = "";
                }
                if (addType === true) {
                    ret = ret + type + ": ";
                }

                var value = String(Math.round(sensor.value * 10) / 10);
                ret = ret + value + " " + unit;
            }
            if (ret.length > 0) {
                arr.push(ret);
            }

            arr.sort();

            ret = "";
            for (s = 0; s < arr.length; s++) {
                ret = ret + arr[s];
                if (s < (arr.length - 1)) {
                    ret = ret + "<br>";
                }
            }

            return ret;
        };

        Ext.Ajax.request(
        {
            url: '/json/property/query',
            method: 'GET',
            scope: this,
            params:
            {
                query: '/usr/states/*(name,value,state)'
            },
            success: function(result) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not load state information from dSS") +
                            ': ' + json.message);
                        this.maskEl.unmask();
                        return;
                    }

                    var system = [];
                    var zones = {};
                    var zoneStore = dSS.data.Loader.getZoneStore();
                    var i;
                    for (i = 0; i < json.result.states.length; i++) {
                        var state = json.result.states[i];
                        if (state.name === "zone.0.group.64.status.malfunction") {
                            var value = "On";
                            if (state.state == "inactive") {
                                value = "Off";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("APT_VENTILATION"),
                                dSS._("Apartment ventilation"),
                                dSS._("Malfunction"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "zone.0.group.64.status.service") {
                            var value = "On";
                            if (state.state == "inactive") {
                                value = "Off";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("APT_VENTILATION"),
                                dSS._("Apartment ventilation"),
                                dSS._("Service"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "holiday") {
                            var value = dSS._("active");
                            if (state.value === "off") {
                                value = dSS._("inactive");
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("PRESENCE_SIM"),
                                dSS._("Presence simulation"),
                                "",
                                value
                            ]);
                        } else if (state.name === "presence") {
                            var value = "Present";
                            if (state.state !== "present") {
                                value = "Absent";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("PRESENCE"),
                                dSS._("Presence in system state table"),
                                "",
                                dSS._(value)
                            ]);
                        } else if ((state.name === "wind") ||
                                   (state.name === "rain") ||
                                   (state.name === "hail") ||
                                   (state.name === "frost")) {
                            var value = state.name;
                            if (state.state === "inactive") {
                                value = "No " + state.name;
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("WEATHER"),
                                dSS._("Weather"),
                                dSS._(state.name),
                                dSS._(value)
                            ]);
                        } else if (state.name === "fire") {
                            var value = state.name;
                            if (state.state === "inactive") {
                                value = "No " + state.name;
                            } else if (state.state === "activeMuted") {
                                value = "Fire muted";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._(state.name),
                                dSS._(value)
                            ]);
                        } else if (state.name === "panic") {
                            var value = state.name;
                            if (state.state === "inactive") {
                                value = "No " + state.name;
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._(state.name),
                                dSS._(value)
                            ]);
                        } else if (state.name === "alarm") {
                            var value = "Alarm 1 active";
                            if (state.state === "inactive") {
                                value = "Alarm 1 inactive";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._("Alarm"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "alarm2") {
                            var value = "Alarm 2 active";
                            if (state.state === "inactive") {
                                value = "Alarm 2 inactive";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._("Alarm"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "alarm3") {
                            var value = "Alarm 3 active";
                            if (state.state === "inactive") {
                                value = "Alarm 3 inactive";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._("Alarm"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "alarm4") {
                            var value = "Alarm 4 active";
                            if (state.state === "inactive") {
                                value = "Alarm 4 inactive";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SECURITY"),
                                dSS._("Security"),
                                dSS._("Alarm"),
                                dSS._(value)
                            ]);
                        } else if (state.name === "daylight") {
                            var value = "Bright";
                            if (state.value === false) {
                                value = "Dark";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SOLAR_ALTITUDE"),
                                dSS._("Solar altitude"),
                                "",
                                dSS._(value)
                            ]);
                        } else if (state.name === "daynight") {
                            var value = "Daytime";
                            if (state.value === false) {
                                value = "Nighttime";
                            }
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("SOLAR_ALTITUDE"),
                                dSS._("Solar altitude"),
                                "",
                                dSS._(value)
                            ]);
                        } else if (state.name === "twilight") {
                            if (state.value === true) {
                                system.push(
                                [
                                    dSS.util.getSystemOverviewSortValue("SOLAR_ALTITUDE"),
                                    dSS._("Solar altitude"),
                                    "",
                                    dSS._("Twilight")
                                ]);
                            }
                        } else if (state.name === "heating_water_system") {
                            system.push(
                            [
                                dSS.util.getSystemOverviewSortValue("HEATING"),
                                dSS._("Heating"),
                                dSS._("Heating system status"),
                                dSS._(state.state)
                            ]);
                        } else if (state.name.lastIndexOf("zone", 0) === 0) {
                            var parts = state.name.split('.');
                            if (parts.length === 3) {
                                var zone_id = parts[1];
                                var type = parts[2];

                                if (state.state !== "unknown") {
                                    var zone = dSS.util.getZoneById(zone_id);
                                    if (type === "light") {
                                        var value = "Light is on";
                                        if (state.state === "inactive") {
                                            value = "Light is off";
                                        }
                                        if (zone) {
                                            if (zones[zone_id] === undefined) {
                                                zones[zone_id] = initZone();
                                            }
                                            zones[zone_id].zone = zone.get('name');
                                            zones[zone_id][dSS.config.groupNames[dSS.config.groups.LIGHT]] = dSS._(value);
                                        }
                                    } else if (type === "motion") {
                                        var value = "Motion";
                                        if (state.state === "inactive") {
                                            value = "No motion";
                                        }
                                        if (zone) {
                                            if (zones[zone_id] === undefined) {
                                                zones[zone_id] = initZone();
                                            }
                                            zones[zone_id].zone = zone.get('name');
                                            if (zones[zone_id].sensors.length > 0) {
                                                zones[zone_id].sensors = zones[zone_id].sensors + dSS._(value);
                                            } else {
                                                zones[zone_id].sensors = dSS._(value);
                                            }
                                        }
                                    } else if (type === "presence") {
                                        // show presence state only when active
                                        if (state.state === "active") {
                                            if (zone) {
                                                if (zones[zone_id] === undefined) {
                                                    zones[zone_id] = initZone();
                                                }
                                                zones[zone_id].zone = zone.get('name');
                                                if (zones[zone_id].sensors.length > 0) {
                                                    zones[zone_id].sensors = zones[zone_id].sensors + "<br>" + dSS._("Present");
                                                } else {
                                                    zones[zone_id].sensors = dSS._("Present");
                                                }
                                            }
                                        }
                                    }
                                }
                            } else if (parts.length === 6) {
                                // zone.xx.group.xx.status.service"
                                var zone_id = parts[1];
                                var group_id = parts[3];
                                var part_id = parts[4];
                                var type = parts[5];

                                if ((part_id == "status") && (state.state !== "unknown")) {
                                    var zone = dSS.util.getZoneById(zone_id);
                                    if (type === "malfunction") {
                                        if (zone) {
                                            if (zones[zone_id] === undefined) {
                                                zones[zone_id] = initZone();
                                            }
                                            zones[zone_id].zone = zone.get('name');

                                            var value = "";
                                            if (state.state === "active") {
                                                value = "Malfunction";
                                            }
                                            if (value.length == 0) {
                                                continue;
                                            }
                                            if (zones[zone_id][dSS.config.groupNames[group_id]].length > 0) {
                                                zones[zone_id][dSS.config.groupNames[group_id]] = zones[zone_id][dSS.config.groupNames[group_id]] + "<br>" +dSS._(value);
                                            } else {
                                                zones[zone_id][dSS.config.groupNames[group_id]] = dSS._(value);
                                            }
                                        }
                                    } else if (type === "service") {
                                        if (zone) {
                                            if (zones[zone_id] === undefined) {
                                                zones[zone_id] = initZone();
                                            }
                                            zones[zone_id].zone = zone.get('name');

                                            var value = "";
                                            if (state.state === "active") {
                                                value = "Service required";
                                            }
                                            if (value.length == 0) {
                                                continue;
                                            }
                                            if (zones[zone_id][dSS.config.groupNames[group_id]].length > 0) {
                                                zones[zone_id][dSS.config.groupNames[group_id]] = zones[zone_id][dSS.config.groupNames[group_id]] + "<br>" +dSS._(value);
                                            } else {
                                                zones[zone_id][dSS.config.groupNames[group_id]] = dSS._(value);
                                            }
                                        }
                                    }
                                }
                            }
                        } else if (state.name.lastIndexOf("wind.group", 0) === 0) {
                            var parts = state.name.split('.');
                            if (parts.length === 2) {
                                var groupId = parts[1].substr("group".length);
                                var group = groupStore.getById(groupId);

                                var value = "No wind in %1";
                                if (state.state === "active") {
                                    value = "Wind in %1";
                                }
                                if (group) {
                                    var name = group.get('name');
                                    system.push(
                                    [
                                        dSS.util.getSystemOverviewSortValue("WEATHER"),
                                        dSS._("Weather"),
                                        dSS._("wind"),
                                        dSS.gt.strargs(dSS._(value),
                                                       [ group.get('name') ])
                                    ]);

                                }
                            }
                        } else if (state.name.lastIndexOf("cluster", 0) === 0) {
                            var parts = state.name.split('.');
                            if (parts.length === 3) {
                                var groupId = parts[1];
                                var type = parts[2];
                                if (type === "operation_lock") {
                                    if (state.state === "active") {
                                        oplocks[groupId] = true;
                                    }
                                }
                            }
                        }
                    }

                    Ext.Ajax.request(
                    {
                        url: '/json/property/query',
                        method: 'GET',
                        scope: this,
                        params:
                        {
                            query: '/apartment/zones/*(ZoneID,name)/groups/*(group,lastCalledScene)/sensor/*(value,type)'
                        },
                        success: function(result) {
                            try {
                                var json = Ext.util.JSON.decode(result.responseText);
                                if (!json.ok) {
                                    Ext.Msg.alert(dSS._("Error"),
                                        dSS._("Could not load state information from dSS") +
                                        ': ' + json.message);
                                    this.maskEl.unmask();
                                    return;
                                }

                                var z;
                                for (z = 0; z < json.result.zones.length; z++) {
                                    var g;
                                    var lz = json.result.zones[z];
                                    if (lz.ZoneID === dSS.config.zones.NEW_DEVICES) {
                                        continue;
                                    }

                                    if (lz.ZoneID === dSS.config.zones.BROADCAST) {
                                        for (g = 0; g < lz.groups.length; g++) {
                                            var grp = lz.groups[g];
                                            if (grp.group == dSS.config.groups.VARIABLE) {
                                                var sar = getSensorsForApartment(grp.sensor);
                                                for (var i = 0; i < sar.length; i++) {
                                                    system.push(
                                                        [
                                                            dSS.util.getSystemOverviewSortValue("WEATHER"),
                                                            dSS._("Weather"),
                                                            dSS._(sar[i].type),
                                                            "",
                                                            dSS._(sar[i].value)
                                                        ]);
                                                }
                                                break;
                                            }
                                        }
                                        continue;
                                    }

                                    for (g = 0; g < lz.groups.length; g++) {
                                        var grp = lz.groups[g];

                                        switch (grp.group)
                                        {
                                            case dSS.config.groups.LIGHT:
                                            case dSS.config.groups.SHADOW:
                                            case dSS.config.groups.HEATING:
                                            case dSS.config.groups.AUDIO:
                                            case dSS.config.groups.VIDEO:
                                            {
                                                var zone = dSS.util.getZoneById(lz.ZoneID);
                                                if (zone) {
                                                    if (zones[lz.ZoneID]
                                                                === undefined) {
                                                        zones[lz.ZoneID] =
                                                                    initZone();
                                                    }
                                                    zones[lz.ZoneID].zone = zone.get('name');
                                                    var lf = "";
                                                    if (zones[lz.ZoneID][dSS.config.groupNames[grp.group]].length > 0) {
                                                        lf = "<br>";
                                                    }
                                                    zones[lz.ZoneID][dSS.config.groupNames[grp.group]] = dSS._(dSS.util.SceneToString(grp.lastCalledScene)) + lf + zones[lz.ZoneID][dSS.config.groupNames[grp.group]];
                                                }
                                                break;
                                            }
                                            case dSS.config.groups.VARIABLE:
                                            {
                                                var sens = getSensors(grp.sensor,
                                                    {
                                                        addType: false,
                                                        ignoreTempCtl: true,
                                                        ignoreSetpoint: true,
                                                        onlySetpoint: false
                                                    });

                                                var zone = dSS.util.getZoneById(
                                                                     lz.ZoneID);
                                                if (zone) {
                                                    if (zones[lz.ZoneID]
                                                            === undefined) {
                                                        zones[lz.ZoneID] =
                                                                initZone();
                                                    }
                                                    zones[lz.ZoneID].zone =
                                                        zone.get('name');
                                                    if (zones[lz.ZoneID].sensors.length > 0) {
                                                        zones[lz.ZoneID].sensors = zones[lz.ZoneID].sensors + "<br>" + sens;
                                                    } else {
                                                        zones[lz.ZoneID].sensors = sens;
                                                    }
                                                }

                                                sens = getSensors(grp.sensor,
                                                    {
                                                        addType: true,
                                                        ignoreTempCtl: true,
                                                        ignoreSetpoint: false,
                                                        onlySetpoint: true
                                                    });
                                                if (zone) {
                                                    if (sens.length > 0) {
                                                        if (zones[lz.ZoneID].tempcontrol.length > 0) {
                                                            zones[lz.ZoneID].tempcontrol = zones[lz.ZoneID].tempcontrol + "<br>" + sens;
                                                        } else {
                                                            zones[lz.ZoneID].tempcontrol = sens;
                                                        }
                                                    }
                                                }

                                                break;
                                            }
                                            case dSS.config.groups.TEMPCONTROL:
                                            {
                                                var str = getSensors(grp.sensor,
                                                    {
                                                        addType: true,
                                                        ignoreTempCtl: false,
                                                        ignoreSetpoint: true,
                                                        onlySetpoint: false
                                                    });
                                                if ((grp.lastCalledScene >= 0) &&
                                                    (grp.lastCalledScene <= 7)) {
                                                    var mode = dSS._(dSS.config.TempControlSceneNames[grp.lastCalledScene]);
                                                    if (str.length > 0) {
                                                        str = str + "<br>" + mode;
                                                    } else {
                                                        str = mode;
                                                    }
                                                }

                                                var zone = dSS.util.getZoneById(
                                                                     lz.ZoneID);
                                                if (zone) {
                                                    if (zones[lz.ZoneID]
                                                            === undefined) {
                                                        zones[lz.ZoneID] =
                                                                initZone();
                                                    }
                                                    zones[lz.ZoneID].zone =
                                                        zone.get('name');
                                                    if (str.length > 0) {
                                                        if (zones[lz.ZoneID].tempcontrol.length > 0) {
                                                            zones[lz.ZoneID].tempcontrol = zones[lz.ZoneID].tempcontrol + "<br>" + str;
                                                        } else {
                                                            zones[lz.ZoneID].tempcontrol = str;
                                                        }
                                                    }
                                                }
                                                break;
                                            }
                                            case dSS.config.groups.VENTILATION:
                                            case dSS.config.groups.RECIRCULATION:
                                            {
                                                var str = "";
                                                if (dSS.config.sceneNamesAV[grp.lastCalledScene] !== undefined) {
                                                    str = dSS._(dSS.config.sceneNamesAV[grp.lastCalledScene]);
                                                }

                                                var zone = dSS.util.getZoneById(
                                                                     lz.ZoneID);
                                                if (zone) {
                                                    if (zones[lz.ZoneID]
                                                            === undefined) {
                                                        zones[lz.ZoneID] =
                                                                initZone();
                                                    }
                                                    zones[lz.ZoneID].zone =
                                                        zone.get('name');
                                                    if (str.length > 0) {
                                                        if (zones[lz.ZoneID][dSS.config.groupNames[grp.group]].length > 0) {
                                                            zones[lz.ZoneID][dSS.config.groupNames[grp.group]] = zones[lz.ZoneID][dSS.config.groupNames[grp.group]] + "<br>" + str;
                                                        } else {
                                                            zones[lz.ZoneID][dSS.config.groupNames[grp.group]] = str;
                                                        }
                                                    }
                                                }
                                                break;
                                            }

                                            default:
                                                break;
                                        }
                                    }
                                }
                                var load = [];
                                for (var property in zones) {
                                    if (zones.hasOwnProperty(property) &&
                                        (property !== '' + dSS.config.zones.NEW_DEVICES) &&
                                        (property !== '' + dSS.config.zones.BROADCAST)) {
                                        load.push(zones[property]);
                                    }
                                }
                                this.zonestates.getStore().loadData({ data: load });
                                this.requestGroupData(oplocks, system);
                            } catch (err) {
                                Ext.Msg.alert(dSS._("Error"),
                                    dSS._("Could not load state information from dSS") +
                                          ": " + err);
                                this.maskEl.unmask();
                            }
                        },
                        failure: function(result) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not load state information from dSS"));
                            this.maskEl.unmask();
                        }
                    });
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not load state information from dSS") +
                              ": " + err);
                    this.maskEl.unmask();
                }
            },
            failure: function(result) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not load state information from dSS"));
                this.maskEl.unmask();
            }
        });
    }
});

Ext.reg('dsssystemstates', dSS.SystemStatesPanel);
