//= require <dSS/ZoneView>
Ext.namespace('dSS');

dSS.ZonePanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.featureConfig = dSS.config.FeatureConfig.ZONE;

        if (config.featureConfig != undefined) {
            this.featureConfig = config.featureConfig;
        }

        Ext.apply(this, {
            title: dSS._("Rooms"),
            layout: 'border',
            items: [
            {
                xtype: 'dsszoneview',
                featureConfig: this.featureConfig,
                ref: 'zoneView',
                region: 'center'
            }]
        });

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.SENSOR)) {
            this.tbar = this.buildTopToolbar();
        }

        this.zoneSortMode = 0;

        dSS.ZonePanel.superclass.constructor.apply(this, arguments);

        this.zoneView.on('selectionchange', function(dv, nodes) {
            var tbar = this.getTopToolbar();
            if (nodes.length > 0) {
                // only single select is allowed
                var selection = dv.getSelectedRecords()[0];
                var isFloor = selection.get('isFloor');

                tbar.blinkButton.setDisabled(isFloor);
                if (isFloor === true) {
                    tbar.renameItemButton.setDisabled(false);
                    tbar.removeZoneButton.setDisabled(isFloor);
                    if (selection.get('floorId') === dSS.config.floors.APARTMENT) {
                        tbar.removeFloorButton.setDisabled(true);
                    } else {
                        tbar.removeFloorButton.setDisabled(false);
                    }
                } else {
                    tbar.removeFloorButton.setDisabled(true);
                    if ((selection.get('zoneId') === dSS.config.zones.NEW_DEVICES) ||
                        (selection.get('zoneId') === dSS.config.zones.BROADCAST)) {
                        tbar.renameItemButton.setDisabled(true);
                        tbar.removeZoneButton.setDisabled(true);
                    } else {
                        tbar.renameItemButton.setDisabled(false);
                        tbar.removeZoneButton.setDisabled(false);
                    }
                }
            } else {
                tbar.blinkButton.setDisabled(true);
                tbar.renameItemButton.setDisabled(true);
                tbar.removeFloorButton.setDisabled(true);
                tbar.removeZoneButton.setDisabled(true);
            }
        }, this);
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'blink',
                ref: 'blinkButton',
                handler: this.blinkZone,
                disabled: true,
                tooltip: dSS._("Blink to identify all devices in room"),
                scope: this
            },
            {
                iconCls: 'addFloorIcon',
                handler: this.createNewFloor,
                tooltip: dSS._("Create new floor"),
                scope: this
            },
            {
                iconCls: 'addZoneIcon',
                handler: this.createNewZone,
                tooltip: dSS._("Create new room"),
                scope: this
            },
            {
                iconCls: 'renameZoneIcon',
                ref: 'renameItemButton',
                handler: this.renameZone,
                disabled: true,
                tooltip: dSS._("Rename selected floor or room"),
                scope: this
            },
            '->',
            {
                iconCls: 'removeFloorIcon',
                ref: 'removeFloorButton',
                handler: this.removeFloor,
                disabled: true,
                tooltip: dSS._("Remove selected floor"),
                scope: this
            },
            {
                iconCls: 'removeZoneIcon',
                ref: 'removeZoneButton',
                handler: this.removeZone,
                disabled: true,
                tooltip: dSS._("Remove selected room"),
                scope: this
            }
        ];
    },
    removeZone: function() {
        this.zoneView.removeZone();
    },
    blinkZone: function() {
        this.zoneView.blinkZone();
    },
    renameZone: function() {
        this.zoneView.renameZone();
    },
    createNewZone: function() {
        this.zoneView.createNewZone();
    },
    removeFloor: function() {
        this.zoneView.removeFloor();
    },
    createNewFloor: function() {
        this.zoneView.createNewFloor();
    }
});

Ext.reg('dsszonepanel', dSS.ZonePanel);

