Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured group)
dSS.devprops.ButtonUsageForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.device === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing device"));
            return;
        }

        this.device = config.device;
        this.group = config.group;
        this.matrix = config.matrix;
        this.dSUID = this.device.get('dsuid');

        this.usage = undefined;

        this.unusedButton = new Ext.Button({
            ref: 'unused',
            enableToggle: true,
            text: dSS._("set to unused"),
            minWidth: 100,
            disabled: (this.group === dSS.config.groups.TEMPCONTROL),
            handler: this.buttonToggled.createDelegate(this)
        });

        this.usedButton = new Ext.Button({
            ref: 'used',
            enableToggle: true,
            text: dSS._("set to used"),
            minWidth: 100,
            disabled: (this.group === dSS.config.groups.TEMPCONTROL),
            handler: this.buttonToggled.createDelegate(this)
        });

        this.addEvents('formsubmitted', 'formloaded');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            height: 72,
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    ref: 'main',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'label',
                            ref: 'current',
                            fieldLabel: dSS._("Current state"),
                            disabled: (this.group === dSS.config.groups.TEMPCONTROL),
                            text: dSS._("loading") + '...'
                        }
                    ],
                    buttonAlign: 'left',
                    buttons:
                    [
                        {
                            xtype: 'box',
                            width: 90
                        },
                        this.unusedButton,
                        this.usedButton
                    ]
                }
            ]
        });

        dSS.devprops.ButtonUsageForm.superclass.constructor.apply(this, arguments);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        if (this.matrix.heatinggroup === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.updateVisibilityByGroup, this);
        }
        
        this.on('beforedestroy', function() {
                this.closed = true;

                dSS.MessageBus.removeListener('inputtypechanged',
                                              this.updateVisibility, this);

                if (this.matrix.heatinggroup === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.updateVisibilityByGroup, this);
                }
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.BUTTON_USAGE) {
                    return;
                }

                if (data.json.ok === true) {
                    this.loadedUsage = data.json.result.buttonUsage;
                    if (this.loadedUsage === "used") {
                        this.main.current.setText(dSS._("Push-button is being used."));
                        this.usedButton.toggle(true, true);
                    } else if (this.loadedUsage === "auto_unused") {
                        this.main.current.setText(dSS._("Push-button has not been used yet (automatic)."));
                    } else if (this.loadedUsage === "manual_unused") {
                        this.main.current.setText(dSS._("Push-button is manually marked as unused."));
                        this.unusedButton.toggle(true, true);
                    } else { 
                        // Its "unsupported", this can happen if the device is
                        // currently in "sensor" mode and has not yet been
                        // reconfigured. We can take auto_unused as default
                        // here, this is what the button will become if
                        // switched to pushbutton mode from sensor if the
                        // user does not apply his own button usage settings
                        // in this dialog
                        this.main.current.setText(dSS._("Push-button has not been used yet (automatic)."));
                    }

                }

                this.fireEvent('formloaded', true);
            },
            this);


        this.on('submit', function() {
                var newUsage = this.loadedUsage;
                var dsuid = this.dSUID;

                if (this.unusedButton.pressed === true) {
                    newUsage = dSS.config.ButtonUsage.MANUAL_UNUSED;
                } else if (this.usedButton.pressed === true) {
                    newUsage = dSS.config.ButtonUsage.USED;
                } else {
                    newUsage = dSS.config.ButtonUsage.AUTO_UNUSED;
                }

                if (newUsage !== this.loadedUsage) {
                    dSS.util.genericRequest({
                        url: '/json/device/setButtonUsage',
                        scope: this,
                        params:
                        {
                            dsuid: dsuid,
                            usage: newUsage
                        },
                        failureMsg: dSS._("Could not set device button usage state."),
                        evt: 'formsubmitted'
                    },
                    function(ok) {
                        if (ok === true) {
                            var rec = dSS.data.Loader.getDeviceStore().getById(dsuid);
                            if (rec !== undefined) {
                                rec.set('buttonUsage', newUsage);
                                rec.commit();
                            }
                        }
                    }
                 );
                } else {
                    this.fireEvent('formsubmitted');
                }
            },
            this);
    },
    buttonToggled: function(button, evt) {
        if (button.pressed == true) {
            if (button.ref == "unused") {
                this.usedButton.toggle(false, true);
            } else if (button.ref == "used") {
                this.unusedButton.toggle(false, true);
            }
            this.fireEvent('formchanged');
        } else {
            // "untoggling" a button is not allowed, so if we get an event 
            // and the button is not in the pressed state, then we need to
            // undo this
            button.toggle(true, true);
        }
    },
    updateVisibility: function(input) {
        if (this.closed == true) {
            return;
        }

        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.hide();
            this.disable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.show();
            this.enable();
        }
    },
    updateVisibilityByGroup: function(group, device) {
        if (this.closed == true) {
            return;
        }

        if (device.get('buttonInputCount') == 0) {
            return;
        }

        if (this.matrix.heatinggroup !== true) {
            return;
        }

        if (group === dSS.config.groups.TEMPCONTROL) {
            this.main.current.disable();
            this.usedButton.toggle(false, true);
            this.unusedButton.toggle(true, true);
            this.main.current.setText(dSS._("Push-button is manually marked as unused."));
            this.usedButton.disable();
            this.unusedButton.disable();
        } else {
            this.usedButton.toggle(false, true);
            this.unusedButton.toggle(false, true);
            this.main.current.setText(dSS._("Push-button has not been used yet (automatic)."));
            this.main.current.enable();
            this.usedButton.enable();
            this.unusedButton.enable();
        }
    }
});

Ext.reg('dssdevbuttonusageform', dSS.devprops.ButtonUsageForm);

