Ext.namespace("dSS", "dSS.config", "dSS.devprops");

dSS.devprops.Loaders = {
    "OUTPUT_MODE"               : 0,
    "COMBINED_PB_OUTMODE"       : 1,
    "SHADE_OPEN_CLOSE"          : 2,
    "SHADE_CLOSE_OPEN"          : 3,
    "SHADE_MTIME_FINS"          : 4,
    "SHADE_DELAY_TIME"          : 5,
    "SHADE_DIR_OF_TRAVEL_RTIME_CH" : 6,
    "SHADE_MTIME_BLINK"         : 7,
    "SHADE_FTEND_TRESHOLDENDSW" : 8, // 0x38 follow up time 0x39 threshold endsw
    "SHADE_NUMST_MTIME_FTUNE"   : 9, // 0x3a num steps 0x3b motion time tune
    "SHADE_CALIBRATION_ALLOWED" : 10,
    "SCENE_CONFIGURATION"       : 11,
    "CONSUMPTION_TRESHOLD_CURRENT" : 12,
    "CONSUMPTION_TRESHOLD_TABLE_ENTRY" : 13,
    "AKM_BINARY_INPUTS"         : 14,
    "AKM_DELAY_CONFIG"          : 15,
    "AKM_INPUT_PROPERTY"        : 16,
    "SCENE_MODE"                : 17,
    "SCENE_VALUE"               : 18,
    "DEVICE_NAME"               : 19,
    "METER_NAME"                : 20,
    "ZONE_NAME"                 : 21,
    "VALVE_PWM_MODE"            : 22,
    "VALVE_CONTROL_MODE"        : 23,
    "VALVE_TIMER_MODE"          : 24,
    "VALVE_TYPE"                : 25,
    "UMV_RELAY"                 : 26,
    "BLINK_CONFIG"              : 27,
    "LOCATION"                  : 28,
    "WIND_PROTECTION"           : 29,
    "OUTPUT_AFTER_IMPULSE"      : 30,
    "SHADE_POS_TIME_MAX"        : 31,
    "OUTPUT_SWITCH_THRESHOLD"   : 32,
    "SENSOR_VALUE"              : 33,
    "TEMPERATURE_OFFSET"        : 34,
    "FTW_MODE"                  : 35,  // visibility of paired dsuid
    "FTW_DISPLAY_SETTINGS"      : 36,
    "FTW_BACKLIGHT_SETTINGS"    : 37,
    "ACTIVE_GROUP"              : 38,
    "SCENE_VALUE_OUTPUT_CHANNELS" : 39,
    "SCENE_VALUE2_OUTPUT_CHANNELS": 40,
    "CONSUMPTION_LED_SETTING"   : 41,
    "DIMTIME0"                  : 42,
    "DIMTIME2"                  : 43,
    "CURRENT_OUTPUT_AUTO_MODE"  : 44,
    "FLOOR_NAME"                : 45,
    "BUTTON_USAGE"              : 46
};

// config options:
// defaultFailMsg: default failure message in case the server does not
//                 reply with anything meaningful
// url: request URL as in Ext.Ajax.request
// params: URL parameters as in Ext.Ajax.request
// scop: objet scope
dSS.devprops.Loader = Ext.extend(Ext.util.Observable, {
    constructor: function(config){
        this.defaultFailMsg = config.defaultFailMsg;
        this.addEvents('loadcomplete');
        this.url = config.url;
        this.loaderId = config.loaderId;
        this.params = config.params;
        this.scope = this;
        if (config.scope !== undefined) {
            this.scope = config.scope;
        }

        this.listeners = config.listeners;

        this.enableBubble('loadcomplete');

        dSS.devprops.Loader.superclass.constructor.call(this, config);
    },

    load: function() {
        // allow dummy loaders
        if (this.url === undefined) {
            var ret = {};
            ret.loader = this.loaderId;
            this.fireEvent('loadcomplete', ret);
            return;
        }
        this.tid = Ext.Ajax.request({
            url: this.url,
            method: 'GET',
            params: this.params,
            scope: this.scope,
            success: function(response) {
                var jsonData;
                var ret = {};
                try {
                    jsonData = Ext.util.JSON.decode(response.responseText);
                } catch (err) {
                    jsonData.ok = false;
                    jsonData.message = this.defaultFailMsg;
                }
                ret.json = jsonData;
                ret.loader = this.loaderId;
                this.fireEvent('loadcomplete', ret);
            },
            failure: function(response) {
                var jsonData;
                var ret = {};
                try {
                    jsonData = Ext.util.JSON.decode(response.responseText);
                } catch (err) {
                    jsonData.ok = false;
                    jsonData.message = this.defaultFailMsg;
                }
                ret.json = jsonData;
                ret.loader = this.loaderId;
                this.fireEvent('loadcomplete', ret);
            }
        });
        return this.tid;
    }
});

Ext.reg('dssdevpropsloader', dSS.devprops.Loader);

