//= require <dSS/devices/device-settings/DeviceSettingsPanel>
//= require <dSS/devices/device-settings/device-type-panels/color-view/Helpers>

/**
 * ScenesPanel > DeviceSettingsRootPanel > DeviceSettingsPanel > DeviceSettingsHSVView | DeviceSettingsFCUView | DeviceSettingsDefaultView ...
 */

Ext.namespace("dSS", "dSS.devices", "dSS.config", "dSS.gthelper");

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsRootPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device;
        if (this.devices.length > 0) {
            device = this.devices[this.index];
        }

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }

        this.embedded = false;
        if (config.embedded === true) {
            this.embedded = true;
        }

        var devCls;
        if (device !== undefined) {
            var devInfo = dSS.util.parseProductID(device.get('productID'));
            devCls = dSS.util.parseFunctionID(device.get('functionID'));
            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(device);
            } catch (error) {
                matrix = error;
            }
        } else {
            try {
                matrix = dSS.devprops.GetVisibleProperties();
            } catch (error) {
                matrix = error;
            }
        }

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.titleIcon = '';
        var enabled = false;
        var devName = " ";
        var devdSUID = "";
        var devVirtualDevice = false;
        var devDisplayID = "";
        var devPresent = false;
        var devClass = 0;

        if (device !== undefined) {
            this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
            enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));
            devName = device.get('name');
            devdSUID = device.get('dsuid');
            devDisplayID = device.get('DisplayID');
            devPresent = device.get('isPresent');
            devVirtualDevice = (device.get('VdcModelUID') != '');
            devClass = devCls[0];

            // output mode dependency, hack for BL-KM
            if (devClass === dSS.config.deviceClass.BL) {
                var devType = dSS.config.deviceTypes[devInfo[0]];
                if ((devType === "KM") && ((devInfo[1] === 200) || (devInfo[1] === 300))) {
                    var hack = dSS.util.getHeatingMods(matrix,
                                                       device.get('outputMode'));
                    matrix = hack.matrix;
                    devClass = hack.deviceClass;
                }

                // #dsd-484 BL-KM200 on 3.6.1 is broken
                if ((device.get('productRevision') == 0x361) &&
                    (device.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {
                    enabled = false;
                }
            }
        }

        var buttons =
        [
            this.left,
            this.right,
            new Ext.Toolbar.Fill()
        ];

        var bodyCss;
        var items = [];
        if (this.embedded === false) {
            buttons.push({
                xtype: 'button',
                text: dSS._("Close"),
                scope: this,
                handler: function() {
                    if (this.maskElement !== undefined) {
                        this.maskElement.unmask();
                    }
                    this.fireEvent('close');
                }
            });
            var info =
            {
                xtype: 'panel',
                ref: 'info',
                border: false,
                items:
                [
                    dSS.util.CreateDefaultDeviceInfoPanel(devName, devDisplayID)
                ]
            };

            items.push(info);
        } else {
            bodyCss = 'x-panel-no-left-border x-panel-no-right-border x-panel-no-top-border';
        }

        items.push(
        {
            xtype: 'dssdevsettingspanel',
            ref: 'settings',
            bodyCssClass: bodyCss,
            dSUID: devdSUID,
            DisplayID: devDisplayID,
            devClass: devClass,
            present: devPresent,
            matrix: matrix,
            virtualDevice: devVirtualDevice,
            device: device
        });

        Ext.apply(this, {
            bubbleEvents: [ 'updatetitleicon', 'close', 'updateshadow',
                            'valuechanged' ],
            layout: 'table',
            bodyBorder: false,
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            items: items,
            buttonAlign: 'left',
            buttons: buttons
        });

        dSS.devices.DeviceSettingsRootPanel.superclass.constructor.apply(this,
                                                                     arguments);
        this.on('afterrender', function() {
            var icon = enabled ? 'loading-indicator-small' : this.titleIcon;
            this.fireEvent('updatetitleicon', icon);
        },
        this);

        this.addEvents('updateshadow', 'updatetitleicon', 'close');

        if (this.maskElement !== undefined) {
            this.maskElement.mask();
        }

        this.on('beforedestroy', function(component) {
            if (this.maskElement !== undefined) {
                this.maskElement.unmask();
            }
        },
        this);

        this.settings.on('loading', function(l) {
            var icon;
            if (l === true) {
                icon = 'loading-indicator-small';
            } else {
                icon = this.titleIcon;
            }
            this.fireEvent('updatetitleicon', icon);
        },
        this);

    },
    setupButtons: function() {
        if (this.index == 0) {
            this.left.disable();
        }

        if ((this.devices.length > 1) &&
            (this.index < (this.devices.length - 1))) {
            this.right.enable();
        }

        if (this.index === (this.devices.length - 1)) {
            this.right.disable();
        }

        if ((this.devices.length > 1) && (this.index > 0)) {
            this.left.enable();
        }

    },
    loadDevices: function(devices) {
        this.devices = devices;
        this.right.disable();
        this.left.disable();
        if (this.devices.length > 0) {
            this.index = 0;
        }
    },
    setRowSelectionModel: function(model) {
        if (this.rowModel !== undefined) {
            this.rowModel.removeListener('rowselect',
                    this.loadRow.createDelegate(this), this);
        }
        this.rowSel = model;
        this.rowSel.addListener('rowselect',
                this.loadRow.createDelegate(this), this);
    },
    clearDialog: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            this.settings.clearData();
            if (!this.embedded) {
                this.info.removeAll(true);
                this.info.add(dSS.util.CreateDefaultDeviceInfoPanel(" ", " "));
                this.info.doLayout();
            }
        }
    },
    loadRow: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            if (record.get('isPresent') === false) {
                return;
            }
            if (this.index < this.devices.length) {
                this.index = index;
            }
            this.setupButtons();
            this.updateView(false);
        }
    },
    updateView: function(doSelect) {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(device);
        } catch (error) {
            matrix = error;
        }

        this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
        this.fireEvent('updatetitleicon', this.titleIcon);

        if (!this.embedded) {
            this.info.removeAll(true);
            this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('DisplayID')));
            this.info.doLayout();
        }

        var devClass = devCls[0];
        // output mode dependency, hack for BL-KM
        if (devClass === dSS.config.deviceClass.BL) {
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if ((devType === "KM") && ((devInfo[1] === 200) || (devInfo[1] === 300))) {
                var hack = dSS.util.getHeatingMods(matrix,
                                                   device.get('outputMode'));
                matrix = hack.matrix;
                devClass = hack.deviceClass;
            }
        }
        this.settings.setData(device.get('dsuid'), devClass,
                              (device.get('isPresent') && (device.get('outputMode') != 0)),
                              matrix, device.get('isVdcDevice'), device);
        if ((this.rowSel !== undefined) && (doSelect !== false)) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

Ext.reg('dssdevicesettingsrootpanel', dSS.devices.DeviceSettingsRootPanel);

dSS.devices.DeviceSettingsWrapper = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.device = config.deviceRecord;
        var devCls = dSS.util.parseFunctionID(this.device.get('functionID'));
        this.matrix = config.visibilityMatrix;

        this.enabled = true;    // "Retain output" checkbox value

        this.scene = config.scene;

        this.resetButton = new Ext.Button({
            text: dSS._("Reset"),
            disabled: true,
            minWidth: 110,
            handler: this.resetSettings.createDelegate(this)
        });

        this.previewButton = new Ext.Button({
            text: dSS._("Preview"),
            minWidth: 110,
            disabled: !((this.device.get('isPresent') === true) &&
                (this.matrix.outvalue8 || this.matrix.shadeposition ||
                this.matrix.outputchannels || this.matrix.fcu)),
            scope: this,
            handler: this.applySettings.createDelegate(this)
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formloaded', 'savebuttonstate', 'formsubmitted',
                            'formchanged' ],
            border: false,
            layout: 'border',
            height: 370,
            items:
            [
                {
                    xtype: 'dssdevsettingspanel',
                    ref: 'devsettings',
                    apartmentMode: true,
                    border: true,
                    bodyCssClass: 'x-panel-no-left-border x-panel-no-right-border x-panel-no-bottom-border',
                    dSUID: this.device.get('dsuid'),
                    DisplayID: this.device.get('DisplayID'),
                    devClass: devCls[0],
                    present: this.device.get('isPresent'),
                    virtualDevice: (this.device.get('VdcModelUID') != ''),
                    matrix: this.matrix,
                    overrideHeight: 340,
                    region: 'north',
                    scene: config.scene,
                    device: this.device
                },
                {
                    xtype: 'panel',
                    region: 'center',
                    layout: 'vbox',
                    border: false,
                    ref: 'buttonpanel',
                    items:
                    [
                        {
                            xtype: 'box',
                            flex: 3
                        },
                        {
                            xtype: 'panel',
                            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 40,
                            layout: 'hbox',
                            border: false,
                            items:
                            [
                                {
                                    xtype: 'box',
                                    width: 10
                                },
                                this.resetButton,
                                {
                                    xtype: 'box',
                                    flex: 3
                                },
                                this.previewButton,
                                {
                                    xtype: 'box',
                                    width: 10
                                }
                            ]
                        },
                        {
                            xtype: 'box',
                            height: 15
                        }
                    ]
                }
            ]
        });

        dSS.devices.DeviceSettingsWrapper.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('formchanged', function() {
            this.previewButton.enable();
            this.resetButton.enable();
        },
        this);

        this.on('enableme', function() {
            if (this.enabled) {
                this.items.each(function(item) {
                    item.enable();
                });
            }
        },
        this, { buffer: 500 });

        this.on('dataloaded', function(data) {
            if ((data.loader !== dSS.devprops.Loaders.SCENE_VALUE) &&
                (data.loader !== dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS) &&
                (data.loader !== dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS)) { 
                return;
            }

            if (data.json.ok !== true) {
                Ext.Msg.alert(dSS._("Error"), data.json.message);
                this.fireEvent('formloaded', false);
                return;
            }

            if (data.loader === dSS.devprops.Loaders.SCENE_VALUE) {
                this.devsettings.sliderView.setDeviceValue(data.json.result.value);

                if (data.json.result.angle !== undefined) {
                    this.devsettings.sliderView.setBladeAngle(data.json.result.angle);
                }
            } else if (data.loader === dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS) {
                var color = data.json.result.channels && data.json.result.channels.reduce(function (allChannels, channel) {
                    allChannels[channel.channel] = channel.channel === 'colortemp'
                        ? Math.round(dSS.devices.helpers.channelValueToTemperature(channel.value))
                        : Math.round(channel.value);
                    return allChannels;
                }, {});
                if (this.matrix.outputchannels === true && color) {
                    this.devsettings.colorPicker.setColor(color);
                }
            } else if (data.loader === dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS) {
                var airflowintensity;
                var louverposition;
                var direction;
                var directiondc;
                var automatic;
                var fandc;
                var swing;
                var louverdc;

                var channels = data.json.result.scenes[this.scene].channels;
                if (channels.airFlowIntensity !== undefined) {
                    var chan = channels.airFlowIntensity;
                    if (chan.dontCare !== undefined) {
                        fandc = chan.dontCare;
                    }
                    if (chan.value !== undefined) {
                        airflowintensity = Math.round(chan.value * 10) / 10;
                    }
                }
                if (channels.airFlowAuto !== undefined) {
                    var chan = channels.airFlowAuto;
                    if (chan.value !== undefined) {
                        automatic = chan.value > 0;
                    }
                }
                if (channels.airLouverPosition !== undefined) {
                    var chan = channels.airLouverPosition;
                    if (chan.dontCare != undefined) {
                        louverdc = chan.dontCare;
                    }
                    if (chan.value != undefined) {
                        louverposition = Math.round(chan.value * 10) / 10;
                    }
                }
                if (channels.airLouverAuto !== undefined) {
                    var chan = channels.airLouverAuto;
                    if (chan.value !== undefined) {
                        swing = chan.value > 0;
                    }
                }

                if (channels.airFlowDirection !== undefined) {
                    var chan = channels.airFlowDirection;
                    if (chan.value !== undefined) {
                        direction = chan.value;
                    }
                    if (chan.dontCare != undefined) {
                        directiondc = chan.dontCare;
                    }
                }

                if (this.matrix.fcu) {
                    if (!isNaN(airflowintensity)) {
                        this.devsettings.fcu.setAirflowIntensity(airflowintensity);
                    }
                    if (!isNaN(louverposition)) {
                        this.devsettings.fcu.setLouverPosition(louverposition);
                    }
                    if (!isNaN(direction)) {
                        this.devsettings.fcu.setAirFlowDirection(direction);
                    }
                    if (automatic !== undefined) {
                        this.devsettings.fcu.setFanAutomatic(automatic);
                    }
                    if (fandc !== undefined) {
                        this.devsettings.fcu.setFanRetainOutput(fandc);
                    }
                    if (swing !== undefined) {
                        this.devsettings.fcu.setLouverSwing(swing);
                    }
                    if (louverdc !== undefined) {
                        this.devsettings.fcu.setLouverRetainOutput(louverdc);
                    }
                    if (directiondc !== undefined) {
                        this.devsettings.fcu.setAirFlowDirectionRetainOutput(directiondc);
                    }
                }
            }
            this.devsettings.resetDefaults();
            this.devsettings.visualConfig(this.matrix, devCls[0],
                                        (this.device.get('VdcModelUID') != ''));

            this.fireEvent('formloaded', true);
        },
        this);

        this.on('submit', function() {
            this.devsettings.saveSceneValues();
            this.resetButton.disable();
            this.fireEvent('formsubmitted');
        },
        this);

        dSS.MessageBus.on('deviceoutputmodechanged', function(checked) {
            // events arrive out of order, dataloaded might override
            // deviceoutputmodechanged and cause a wrong dialog status
            this.enabled = !checked;

            // can't say why but using 'this' object produced strange Ext
            // errors, tried to disable items separately, i.e. via
            // this.devsettings - but 'devsettings' object got lost from time
            // to time, which I also can't explain.
            // The workaround below seems to do the trick.
            if (checked) {
                this.items.each(function(item) {
                    item.disable();
                });
            } else {
                this.items.each(function(item) {
                    item.enable();
                });
            }
        },
        this);
    },
    applySettings: function() {
        this.devsettings.doPreview();
        this.previewButton.disable();
    },
    resetSettings: function() {
        this.devsettings.resetValues();
        this.previewButton.enable();
        this.resetButton.disable();
    }
});

Ext.reg('dssdevicesettingswrapper', dSS.devices.DeviceSettingsWrapper);

