Ext.namespace("dSS", "dSS.config", "dSS.devprops");
        // configuration values are set up and saved from the advanced form,
        // the standard form gives hints to the advanced form configuration

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.GroupPushbuttonForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }
        this.dSUID = config.dSUID;
        this.group = config.group;
        this.product = config.product;
        this.number = config.productNumber;

        if (config.matrix === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("GroupPushbuttonForm: " +
                                                  "missing matrix"));
            return;
        }
        this.matrix = config.matrix;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        if (this.matrix.pushbarea === true) {
            this.areaSelection = new Ext.form.ComboBox({
                typeAhead: false,
                triggerAction: 'all',
                lazyRender:true,
                forceSelection: true,
                editable: false,
                width: 100,
                x: 255,
                y: 0,
                mode: 'local',
                store: new Ext.data.ArrayStore({
                    id: 0,
                    fields:
                    [
                        'id',
                        'displayText'
                    ],
                    data:
                    [
                        [dSS.devprops.PBConfig.PUSHBUTTON_AREA1, dSS._("Area 1")],
                        [dSS.devprops.PBConfig.PUSHBUTTON_AREA2, dSS._("Area 2")],
                        [dSS.devprops.PBConfig.PUSHBUTTON_AREA3, dSS._("Area 3")],
                        [dSS.devprops.PBConfig.PUSHBUTTON_AREA4, dSS._("Area 4")]
                    ]
                }),
                valueField: 'id',
                displayField: 'displayText'
            });
        }

        var defaultHeight = 25;
        var pbComboFieldLabel = dSS._("Pushbutton");
        var pbData = [];

        if (this.group !== dSS.config.groups.JOKER) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                    dSS._("zone pushbutton")
                ]
            ];

            if ((config.matrix.pushbarea === true) &&
                ((this.group === dSS.config.groups.LIGHT) ||
                 (this.group === dSS.config.groups.SHADOW))) {
                pbData =
                [
                    [
                        dSS.devprops.PBConfig.PUSHBUTTON_AREA,
                        dSS._("area pushbutton")
                    ]
                ].concat(pbData);
            }
        }

        if (config.matrix.pushbdevice === true) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_DEVICE,
                    dSS._("device pushbutton")
                ]
            ].concat(pbData);
        }

        if ((config.matrix.highlevel === true) &&
            (this.group === dSS.config.groups.JOKER)) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                    dSS._("server button")
                ]
            ].concat(pbData);
        }

        if (this.group === dSS.config.groups.JOKER) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME,
                    dSS._("leaving home")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL,
                    dSS._("door bell")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_PANIC,
                    dSS._("panic")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_ALARM,
                    dSS._("alarm")
                ]
            ].concat(pbData);

            if (config.matrix.pushbsensor === true) {
                pbComboFieldLabel = dSS._("Input mode");
                defaultHeight = 30;
                if (config.matrix.enabletypehack !== true) {
                    pbData.push(
                    [
                        dSS.devprops.PBConfig.PUSHBUTTON_SENSOR,
                        dSS._("sensor")
                    ]);
                }
            }
        }

        if (config.matrix.pushbdisabled === true) {
            pbData.push(
            [
                dSS.devprops.PBConfig.PUSHBUTTON_DISABLED,
                dSS._("disabled")
            ]);
        }

        var devStore = dSS.data.Loader.getDeviceStore();
        var groupStore = dSS.data.Loader.getGroupStore();
        var rec = devStore.getById(this.dSUID);
        if (rec) {
            var groups = rec.get('groups');
            Ext.each(groups, function(group) {
                if (!dSS.util.isUserGroup(group)) {
                    return true;
                }

                var gr = groupStore.getById(group);
                if (gr !== undefined) {
                    pbData.push([ group,
                                  gr.get('name') + " (" + dSS._("Group") + ")",
                                  gr.get('color')]);
                }
            });
        }

        this.pbCombo = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            fieldLabel: pbComboFieldLabel,
            mode: 'local',
            disabled: (this.group === dSS.config.groups.TEMPCONTROL),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText',
                    'color'
                ],
                sortInfo:
                {
                    field: 'displayText',
                    direction: 'asc'
                },
                data: pbData
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        var pbAreaItems =
        [
            {
                xtype: 'panel',
                border: false,
                width: 355,
                x: 0,
                y: 0,
                layout: 'form',
                items: [ this.pbCombo ]
            }
        ];

        if (this.matrix.pushbarea === true) {
            pbAreaItems = pbAreaItems.concat(this.areaSelection);
        }

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            height: 35,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: defaultHeight,
                    width: 355,
                    layout: 'absolute',
                    items: pbAreaItems
                }
            ]
        });

        dSS.devprops.GroupPushbuttonForm.superclass.constructor.apply(this, arguments);
        this.on('loadcomplete', function() {
            if (this.matrix.pushbarea === true) {
                this.areaSelection.on('select', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
            }

            this.pbCombo.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        dSS.MessageBus.addListener('jokerconfigformchanged',
                this.saveJokerValue, this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.pbCombo.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var modSelection = function(val) {
                    var store = this.pbCombo.getStore();
                    var selrec = store.getById(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                    if (selrec !== undefined) {
                        store.remove(selrec);
                    }
                    selrec = store.getById(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                    if (selrec === undefined) {
                        if (val) {
                            store.on('load', function() {
                                this.pbCombo.setValue(val);
                                store.singleSort('displayText', 'asc');
                            }, this, { single: true });
                        }
                        store.loadData(
                            [
                                [
                                    dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                                    dSS._("server button")
                                ]
                            ], true);
                    } else {
                        if (val) {
                            this.pbCombo.setValue(val);
                        }
                    }
                }.createDelegate(this);

                var val = data.json.result[this.dSUID][0].button[0].id;
                var activeGroup =
                    data.json.result[this.dSUID][0].button[0].activeGroup;
                try {
                    var dec = this.decodeConfig(val);
                    if (dSS.util.isUserGroup(activeGroup)) {
                        dec[0] = activeGroup;
                        dec[1] = undefined;
                    }

                    this.pbCombo.setValue(dec[0]);
                    // preset hack for some fw that was missing defaults
                    if ((this.matrix.highlevel === true) &&
                        (this.group == dSS.config.groups.JOKER) &&
                        ((dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_PANIC) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_ALARM) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME))) {
                        dec[0] = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                        this.pbCombo.setValue(dec[0]);
                    }
                    if (dec[0] == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        if (this.matrix.pushbarea === true) {
                            this.areaSelection.setValue(dec[1]);
                        }
                    } else if (dec[0] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                        modSelection(dec[0]);
                    }
                    if (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        if (this.matrix.pushbarea === true) {
                            this.areaSelection.disable();
                        }
                    }
                    if (this.group == dSS.config.groups.JOKER) {
                        if ((this.product == dSS.config.deviceTypeNum['TKM']) ||
                            (this.product == dSS.config.deviceTypeNum['KL']) ||
                            (this.product == dSS.config.deviceTypeNum['ZWS'])) {
                            modSelection();
                        }
                        else if (this.product == dSS.config.deviceTypeNum['UMR']) {
                            var lt = data.json.result[this.dSUID][0].button[0].inputMode;
                            if ((lt >= dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) &&
                                (lt <= dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE)) {
                                this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_SENSOR);
                            }
                        }
                    }

                    if (this.matrix.pushbdisabled === true) {
                        var lt = data.json.result[this.dSUID][0].button[0].inputMode;
                        if (lt == dSS.config.deviceParams.BUTTONINPUT_DISABLED) {
                            this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_DISABLED);
                        }
                    }
                    // #2288: an invalid button configuration will have no
                    // associated text, it will just be a number, so the
                    // simpliest check is: if the combo selection text evaluates
                    // to a valid number - then the setting is invalid
                    var textValue = this.pbCombo.getRawValue();
                    if (!isNaN(parseInt(textValue, 10))) {
                        this.pbCombo.setValue(
                                this.pbCombo.getStore().getAt(0).get('id'));
                        this.on('loadcomplete', function() {
                            this.fireEvent('formchanged');
                        }, this, { single: true });
                    }
                } catch (ex) {
                    if (this.closed === true) {
                        return;
                    }

                    this.pbCombo.markInvalid(ex);
                }
                this.fireEvent('formloaded', true);
                var setting = this.pbCombo.getValue();

                if (this.matrix.pushbarea === true) {
                    if (setting === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        setting = this.areaSelection.getValue();
                    }
                }

                dSS.MessageBus.fireEvent('pushbuttonstdformchanged',
                                          this.pbCombo.getValue());

            },
            this);

        if (this.matrix.highlevel === true) {
            dSS.MessageBus.addListener('jokerconfigformchanged',
                                  this.handleOptionalButtons, this);
        }

        if (this.matrix.heatinggroup === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.handlePBconfig, this);
        }

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('beforedestroy', function() {
                this.closed = true;
                dSS.MessageBus.removeListener('inputtypechanged',
                                              this.updateVisibility, this);

                if (this.matrix.highlevel === true) {
                    dSS.MessageBus.removeListener('jokerconfigformchanged',
                                           this.handleOptionalButtons, this);
                }

                dSS.MessageBus.removeListener('jokerconfigformchanged',
                        this.saveJokerValue, this);

                if (this.matrix.heatinggroup === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.handlePBconfig, this);
                }
            },
            this);

        if (this.matrix.pushbarea === true) {
            this.areaSelection.on('select', function(combo) {
                var v = combo.getValue();
                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);
        }

        this.pbCombo.on('select', this.recalcPB.createDelegate(this), this);

        this.pbCombo.on('beforeselect', function(combo, record) {
                if (this.currentJokerValue === undefined) {
                    return true;
                }

                var color = record.get('color');
                if (isNaN(parseInt(color, 10))) {
                    return true;
                }

                if (this.currentJokerValue !== color) {
                    return false;
                }

                return true;
            },
            this);

        this.on('submit', function() {
            // data will be saved by advanced form
            this.fireEvent('formsubmitted');
        },
        this);
    },

    saveJokerValue: function(color) {
        this.currentJokerValue = color;

        var record = this.pbCombo.getStore().getById(this.pbCombo.getValue());
        if (record !== undefined) {
            var selectedGroupColor = record.get('color');
            if (!isNaN(parseInt(selectedGroupColor, 10))) {
                if (selectedGroupColor != color) {
                    var selRec = this.pbCombo.getStore().getAt(0);
                    if (selRec !== undefined) {
                        this.pbCombo.setValue(selRec.get('id'));
                    }
                }
            }
        }

        if (this.matrix.jokertempcontrol === true) {
            this.handlePBconfig(color);
        }
    },

    recalcPB: function(combo) {
        var v = combo.getValue();
        if (this.matrix.pushbarea === true) {
            if (v == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                if (this.areaSelection.getRawValue() == "") {
                    this.areaSelection.setValue(dSS.devprops.PBConfig.PUSHBUTTON_AREA1);
                }

                v = this.areaSelection.getValue();
                this.areaSelection.enable();
            } else {
                this.areaSelection.setRawValue("");
                this.areaSelection.disable();
            }
        }
        dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
    },

    handlePBconfig: function(group, device) {
        if (device.get('buttonInputCount') == 0) {
            return;
        }

        if ((this.matrix.heatinggroup !== true) &&
            (this.matrix.jokertempcontrol !== true)) {
            return;
        }

        if (group === dSS.config.groups.TEMPCONTROL) {
            this.pbCombo.disable();
        } else {
            this.pbCombo.enable();
        }
    },

    handleOptionalButtons: function(color) {
        if ((this.matrix.highlevel !== true) &&
            (this.matrix.pushbsensor !== true)) {
            return;
        }

        var val = this.pbCombo.getValue();
        var store = this.pbCombo.getStore();

        var rmRecord = function(id) {
            var rec = store.getById(id);
            if (rec !== undefined) {
                store.remove(rec);
            }
        }.createDelegate(this);

        var addRecord = function(id, text) {
            var rec = store.getById(id);
            if (rec === undefined) {
                store.on('load', function() {
                    store.singleSort('displayText', 'asc');
                }, this, { single: true });
                store.loadData([ [ id, text ] ], true);
            }
        }.createDelegate(this);

        if (color === dSS.config.groups.JOKER) {
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                      dSS._("server button"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME,
                      dSS._("leaving home"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL,
                      dSS._("door bell"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_PANIC,
                      dSS._("panic"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_ALARM,
                      dSS._("alarm"));
            if (this.matrix.pushbsensor === true) {
                if (this.matrix.enabletypehack !== true) {
                    addRecord(dSS.devprops.PBConfig.PUSHBUTTON_SENSOR,
                              dSS._("sensor"));
                }
            }

            if (val === dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
                this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
            } else if (val === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                if (this.matrix.pushbdevice === true) {
                    this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_DEVICE);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
                } else {
                    this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
                }
            }
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_AREA);

            if (this.matrix.pushbarea === true) {
                this.areaSelection.setRawValue("");
                this.areaSelection.disable();
            }
        } else {
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_PANIC);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_ALARM);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL);
            if (this.matrix.pushbsensor === true) {
                rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_SENSOR);
            }

            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                      dSS._("zone pushbutton"));

            if ((this.matrix.pushbarea === true) &&
                ((color === dSS.config.groups.LIGHT) ||
                 (color === dSS.config.groups.SHADOW))) {
                addRecord(dSS.devprops.PBConfig.PUSHBUTTON_AREA,
                          dSS._("area pushbutton"));
            } else {
                rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_AREA);
                this.areaSelection.setRawValue("");
                this.areaSelection.disable();
                if (val === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                    this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
                }
            }

            if ((val === dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_PANIC) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_ALARM) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_SENSOR) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME)) {
                this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
            }
        }

        var setting = this.pbCombo.getValue();
        if (this.matrix.pushbarea === true) {
            if (setting === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                setting = this.areaSelection.getValue();
            }
        }
        dSS.MessageBus.fireEvent('pushbuttonstdformchanged', setting);
    },

    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var rbval;
        var ext;
        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ALARM:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_ALARM;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_PANIC:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_PANIC;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_APARTMENT:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");
        }
        return [ rbval, ext ];
    },
    updateVisibility: function(input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.hide();
            this.disable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.show();
            this.enable();
        }
        this.recalcPB(this.pbCombo);
        this.fireEvent('updateshadow');
    }

});

Ext.reg('dssdevgrouppushbuttonform', dSS.devprops.GroupPushbuttonForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.GroupPushbuttonAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;
        this.dSUID = this.device.get('dsuid');
        this.matrix = config.matrix;
        this.group = config.group;
        this.loadedGroup = config.group;
        this.product = config.product;

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];        

        this.AKMInputValue = undefined;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.zonePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS._("Scenes 0-4"),
                dSS._("Scenes 2-4"),
                dSS._("Scene 1")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS._("Scenes 10-14"),
                dSS._("Scenes 12-14"),
                dSS._("Scene 11")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS._("Scenes 20-24"),
                dSS._("Scenes 22-24"),
                dSS._("Scene 21")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS._("Scenes 30-34"),
                dSS._("Scenes 32-34"),
                dSS._("Scene 31")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS._("Scenes 40-44"),
                dSS._("Scenes 42-44"),
                dSS._("Scene 41")
            ]
        ];


        if (this.matrix.pushbarea === true) {
            this.areaPB =
            [
                // area 1
                [
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                        dSS._("Scenes 2-4 (default)"),
                        dSS._("Scenes 2-4 (default)"),
                        dSS._("Area 1")
                    ],
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                        dSS._("Scenes 12-14"),
                        dSS._("Scenes 12-14"),
                        dSS._("Area 1")
                    ]
                ],
                // area 2
                [
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                        dSS._("Scenes 2-4"),
                        dSS._("Scenes 2-4"),
                        dSS._("Area 2")
                    ],
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                        dSS._("Scenes 22-24"),
                        dSS._("Scenes 22-24"),
                        dSS._("Area 2")
                    ]
                ],
                // area 3
                [
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                        dSS._("Scenes 2-4"),
                        dSS._("Scenes 2-4"),
                        dSS._("Area 3")
                    ],
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                        dSS._("Scenes 32-34"),
                        dSS._("Scenes 32-34"),
                        dSS._("Area 3")
                    ]
                ],
                // area 4
                [
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                        dSS._("Scenes 2-4"),
                        dSS._("Scenes 2-4"),
                        dSS._("Area 4")
                    ],
                    [
                        dSS.config.deviceParams.PUSHBUTTON_GROUP8,
                        dSS._("Scenes 42-44"),
                        dSS._("Scenes 42-44"),
                        dSS._("Area 4")
                    ]
                ]
            ];
        }

        this.devicePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS._("Scenes 2-4"),
                dSS._("Scenes 2-4"),
                dSS._("Device")

            ]
        ];

        this.highlevelPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL, "", "", ""
            ]
        ];

        this.panicPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_PANIC, "", "", ""
            ]
        ];

        this.leavePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME, "", "", ""
            ]
        ];

        this.bellPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL, "", "", ""
            ]
        ];

        this.alarmPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ALARM, "", "", ""
            ]
        ];

        this.sensorPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_SENSOR, "", "", ""
            ]
        ];

        this.sceneSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            autoSelect: false,
            editable: false,
            width: 170,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText',
                    'label2Text',
                    'label1Text'
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            fieldLabel: dSS._("Used scenes")
        });

        this.label1 = new Ext.form.Label({
             xtype: 'label',
             fieldLabel: dSS._("1x Click")
         });

        this.label2 = new Ext.form.Label({
            xtype: 'label',
            fieldLabel: dSS._("2x, 3x, 4x Click")
        });

        this.localPrio = new Ext.form.Checkbox({
            boxLabel: dSS._("Local priority for 1x click")
        });

        this.callsPresent = new Ext.form.Checkbox({
            boxLabel: dSS._("Calls Present on click")
        });

        this.lpTip = new Ext.ToolTip({
            html: dSS._("The local priority settings were changed."),
            title: dSS._("Notice"),
            closable: true,
            draggable: true,
            dismissDelay: 4000,
            anchor: 'left'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'formsetdeviceconfig' ],
            padding: 5,
            border: false,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    ref: 'main',
                    height: 140,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Pushbutton") + ':'
                        },
                        this.sceneSelection,
                        this.label1,
                        this.label2,
                        this.localPrio,
                        this.callsPresent
                    ]
                }
            ]
        });

        dSS.devprops.GroupPushbuttonAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.callsPresent.on('check', function() {
                    this.fireEvent('formchanged');
                },
                this);

                this.localPrio.on('check', function() {
                    this.fireEvent('formchanged');
                },
                this);

                this.sceneSelection.on('select', function() {
                        this.fireEvent('formchanged');
                },
                this);
            },
            this);

        dSS.MessageBus.addListener('pushbuttonstdformchanged',
                          this.updateForm, this);

        dSS.MessageBus.addListener('jokerconfigformchanged',
                          this.updateLocalPriority, this);


        dSS.MessageBus.addListener('akminputchanged', this.saveAKMInput, this);

        if (this.matrix.heatinggroup === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.handleAdvancedView, this);
        }

        if (this.matrix.jokertempcontrol === true) {
            dSS.MessageBus.addListener('jokerconfigformchanged',
                this.handleAdvancedView, this);
        }

        this.sceneSelection.on('select', function(combo, record, index) {
                this.label1.setText(record.get('label1Text'));
                this.label2.setText(record.get('label2Text'));
            },this);

        this.loadedValue = undefined;
        this.loadedPrioLocalScene = undefined;
        this.loadedCallsPresent = undefined;
        this.currentValue = undefined;
        this.loadedActiveGroup = undefined;
        this.activeGroup = undefined;

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }
                if (data.loader == dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.callsPresent.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        var val = data.json.result[this.dSUID][0].button[0].id;
                        var prio = data.json.result[this.dSUID][0].button[0].setsLocalPriority;
                        var callsPres = data.json.result[this.dSUID][0].button[0].callsPresent;
                        this.loadedActiveGroup =
                           data.json.result[this.dSUID][0].button[0].activeGroup;
                        this.activeGroup = this.loadedActiveGroup;

                        var load = this.decodeConfig(val);
                        this.loadedValue = load[1];

                        // preset hack for fw without defaults
                        if ((this.matrix.highlevel === true) &&
                            (this.group == dSS.config.groups.JOKER) &&
                            ( ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_PANIC) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_ALARM) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_DEVICE))) {
                            val = dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL;
                            load = this.decodeConfig(val);
                        }

                        this.sceneSelection.getStore().on('load', function() {
                            this.sceneSelection.setValue(load[1]);
                            this.label1.setText(load[2]);
                            this.label2.setText(
                            this.sceneSelection.getStore().getById(load[1]).get(
                                                                    'label2Text'));
                        }, this, { single: true });

                        this.sceneSelection.getStore().loadData(load[0]);
                        // do not overwrite, could have been set by updateForm
                        // already
                        if (this.currentValue === undefined) {
                            this.currentValue = load[4];
                        }
                        this.loadedPrioAreas = prio;
                        this.localPrio.setValue(prio);
                        this.loadedPrioLocalScene = prio;
                        this.localPrio.setDisabled(!load[3]);
                        this.loadedCallsPresent = callsPres;
                        this.callsPresent.setValue(callsPres);
                        this.callsPresent.setDisabled(false);

                        if (load[4] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                            this.disableElements();
                        }

                        if (this.group == dSS.config.groups.JOKER) {
                            if (this.product == dSS.config.deviceTypeNum['UMR']) {
                                var lt = data.json.result[this.dSUID][0].button[0].inputMode;
                                if ((lt >= dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) &&
                                    (lt <= dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE)) {
                                    this.callsPresent.setValue(false);
                                    this.callsPresent.disable();;
                                    this.disableElements();
                                }
                            }
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }

                        this.sceneSelection.markInvalid(ex);
                        this.label1.setText("");
                        this.label2.setText("");
                    }
                } else if (data.loader == dSS.devprops.Loaders.SCENE_CONFIGURATION) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.callsPresent.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        this.loadedPrioLocalScene = data.json.result.specialMode;
                        if (this.loadedValue === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                            this.localPrio.setValue(this.loadedPrioLocalScene ? 1 : 0);
                            this.localPrio.setDisabled(false);
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }
                    }
                }

                if ((this.loadedValue !== undefined) &&
                    (this.loadedPrioLocalScene !== undefined)) {
                    this.fireEvent('formloaded', true);
                }
            },
            this);

		dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('beforedestroy', function() {
				dSS.MessageBus.removeListener('inputtypechanged',
                                              this.updateVisibility, this);
                dSS.MessageBus.removeListener('pushbuttonstdformchanged',
                                              this.updateForm, this);
                dSS.MessageBus.removeListener('jokerconfigformchanged',
                                              this.updateLocalPriority, this);
                dSS.MessageBus.removeListener('akminputchanged',
                                              this.saveAKMInput, this);

                if (this.matrix.heatinggroup === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.handleAdvancedView, this);
                }
                if (this.matrix.jokertempcontrol === true) {
                     dSS.MessageBus.removeListener('jokerconfigformchanged',
                        this.handleAdvancedView, this);
                }

                this.closed = true;
            },
            this);

        this.on('submit', function() {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            var value = this.sceneSelection.getValue();
            var prio = this.localPrio.getValue();
            var callsPresent = this.callsPresent.getValue();

            var id = this.dSUID;
            var deferDelay = 1000;
            var eventSent = false;

            // UMR hack
            if ((!value) && (this.AKMInputValue)) {
                this.ltmode = this.AKMInputValue;
            }

            var setButtonConfig = (this.ltmode !== undefined);

            if (setButtonConfig === true) {
                var rec = dSS.data.Loader.getDeviceStore().getById(id);
                if (rec !== undefined) {
                    if (rec.get('buttonInputMode') === this.ltmode) {
                        setButtonConfig = false;
                    }
                }
            }

            if (this.matrix.setumr200config === true) {
                var parameter = {};

                if (setButtonConfig === true) {
                    parameter.stateMode = dSS.config.ButtonInputModeV2Map[this.ltmode];
                }

                if (((value !== undefined) && (String(value).length > 0)) &&
                    ((this.group != this.loadedGroup) ||
                    ((value !== this.loadedValue) ||
                    ((this.activeGroup === dSS.config.ResetActiveButtonGroup) ||
                     (!dSS.util.isUserGroup(this.activeGroup)))))) {

                    var map;
                    if (this.group === dSS.config.groups.JOKER) {
                        map = dSS.config.PushButtonV2JokerMap[value];
                    } else {
                        map = dSS.config.PushButtonV2Map[value];
                    }
                    parameter.pushButtonTarget = map.target;

                    if (map.area !== undefined) {
                        parameter.pushButtonTargetArea = map.area;
                    }
                    if (map.preset !== undefined) {
                        parameter.pushButtonTargetPresets = map.preset;
                    }
                }

                if (!((value !== this.loadedValue) ||
                      (this.activeGroup !== this.loadedActiveGroup)) &&
                    !(callsPresent != this.loadedCallsPresent) &&
                    !((value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioLocalScene)) &&
                    !((value !== dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioAreas))) {
                    if (Object.keys(parameter).length > 0) {
                        parameter.state = "pushButton"

                        this.fireEvent('formsetdeviceconfig', parameter);
                    } else {
                        this.fireEvent('formsubmitted');
                    }
                    return;
                }

                if ((dSS.util.isUserGroup(this.activeGroup) ||
                    (this.activeGroup === dSS.config.ResetActiveButtonGroup)) &&
                    (this.activeGroup !== this.loadedActiveGroup)) {
                    eventSent = true;

                    if (!dSS.util.isUserGroup(this.activeGroup) &&
                        (this.activeGroup < this.loadedActiveGroup)) {
                        this.activeGroup = dSS.config.ResetActiveButtonGroup;
                    }
                    parameter.pushButtonTargetClusterId = this.activeGroup;
                }

                // hack for GE-TKM300 which outputs should be configured differently
                // than for other devices with setumr200config === true in the matrix:
                var deviceIsGETKM300 = rec && rec.get instanceof Function && rec.get('HW') === 'GE-TKM300'; 

                // UMR can not be configured as device pushbutton, so
                // the setSceneMode call is not needed
                // (unless it's GE-TKM300 which is an exception, see additional request below)
                if (value !== dSS.config.deviceParams.PUSHBUTTON_DEVICE || deviceIsGETKM300) {
                    parameter.pushButtonLocalPriorityFor1xClick = prio;
                }

                if (callsPresent != this.loadedCallsPresent) {
                    parameter.pushButtonCallsPresent = callsPresent;
                }

                var pushButtonConfigDirty = Object.keys(parameter).length > 0;
                var devicePushButtonGETKM300LocalPrioDirty = 
                    deviceIsGETKM300 &&
                    value === dSS.config.deviceParams.PUSHBUTTON_DEVICE &&
                    prio != this.loadedPrioLocalScene;
                if (pushButtonConfigDirty) {
                    this.fireEvent('formsetdeviceconfig', parameter);
                }
                // GE-TKM300 requires additional request to store localPrio,
                // compared to other devices with setumr200config === true
                if (devicePushButtonGETKM300LocalPrioDirty) {
                    dSS.util.genericRequest.defer(deferDelay, this, [ {
                        url: '/json/device/setSceneMode',
                        params: {
                            sceneID : dSS.config.scenes.LOCAL_ON,
                            specialMode: prio ? 1 : 0,
                            dsuid: this.dSUID
                        },
                        failureMsg: dSS._("Could not save pushbutton settings"),
                        scope: this
                    } ]);
                }
                if (!pushButtonConfigDirty) {
                    this.fireEvent('formsubmitted');
                }
                return;
            }

            if (setButtonConfig === true) {
                eventSent = true;
                var ltmode = this.ltmode; // scoping

                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setConfig',
                            params:
                            {
                                dsuid : this.dSUID,
                                'class': 3,
                                index: 0x1e, // ltmode
                                value: this.ltmode
                            },
                            failureMsg:
                                    dSS._("Could not save button settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var rec = dSS.data.Loader.getDeviceStore().getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonInputMode', ltmode);
                                    rec.commit();
                                }
                            }
                        }
                    ]
                );
                deferDelay += 500;
            }

            if (!((value !== this.loadedValue) ||
                  (this.activeGroup !== this.loadedActiveGroup)) &&
                !(callsPresent != this.loadedCallsPresent) &&
                !((value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioLocalScene)) &&
                !((value !== dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioAreas))) {
                this.fireEvent('formsubmitted');
                return;
            }

            if (((value !== undefined) && (String(value).length > 0)) &&
                ((value !== this.loadedValue) ||
                ((this.activeGroup === dSS.config.ResetActiveButtonGroup) ||
                 (!dSS.util.isUserGroup(this.activeGroup))))) {
                eventSent = true;
                // we know this form is never used alone, it's always in the
                // same dialog as the output mode form; in order to not spam
                // the bus with two simultanious requests we will delay saving
                // of pushbutton settings by one second
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setButtonID',
                            params:
                            {
                                dsuid : this.dSUID,
                                buttonID : value
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var rec = dSS.data.Loader.getDeviceStore().getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonID', value);
                                    rec.commit();
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        }
                    ]
                );
                deferDelay += 500;
            }

            if ((dSS.util.isUserGroup(this.activeGroup) ||
                (this.activeGroup === dSS.config.ResetActiveButtonGroup)) &&
                (this.activeGroup !== this.loadedActiveGroup)) {
                eventSent = true;

                if (!dSS.util.isUserGroup(this.activeGroup) &&
                    (this.activeGroup < this.loadedActiveGroup)) {
                    this.activeGroup = dSS.config.ResetActiveButtonGroup;
                }
                var ag = this.activeGroup;
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setButtonActiveGroup',
                            params:
                            {
                                dsuid : this.dSUID,
                                groupID : this.activeGroup
                            },
                            failureMsg:
                                    dSS._("Could not save user group settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success, json) {
                            if (success === true) {
                                dSS.util.processDeviceActionResponse(json.result, false);
                            }
                        }
                    ]
                );
                deferDelay += 500;
            }

            if (value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                if (prio != this.loadedPrioLocalScene) {
                    eventSent = true;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/device/setSceneMode',
                                params:
                                {
                                    sceneID : dSS.config.scenes.LOCAL_ON,
                                    specialMode: prio ? 1 : 0,
                                    dsuid: this.dSUID
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            } else {
                if ((this.localPrio.disabled !== true) &&
                    (prio != this.loadedPrioAreas)) {
                    eventSent = true;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/property/setBoolean',
                                params:
                                {
                                    path: '/apartment/zones/zone0/devices/' +
                                          this.dSUID +
                                          '/button/setsLocalPriority',
                                    value: prio
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            }
            if (callsPresent != this.loadedCallsPresent) {
                eventSent = true;
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/property/setBoolean',
                            params:
                            {
                                path: '/apartment/zones/zone0/devices/' +
                                      this.dSUID +
                                      '/button/callsPresent',
                                value: callsPresent
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        }
                    ]
                );
            }

            if (eventSent !== true) {
                this.fireEvent('formsubmitted');
            }
        },
        this);
    },
    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var ret;
        var l1;
        var showLocalPriority = false;
        var rbval;

        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                ret = this.devicePB;
                l1 = dSS._("Device");
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ALARM:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_ALARM;
                    ret = this.alarmPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                if (this.matrix.pushbarea === true) {
                    ret = this.areaPB[0];
                    l1 = dSS._("Area 1");
                    showLocalPriority = true;
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                }
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_PANIC;
                    ret = this.panicPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                if (this.matrix.pushbarea === true) {
                    ret = this.areaPB[1];
                    l1 = dSS._("Area 2");
                    showLocalPriority = true;
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                }
                break;
            case dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME;
                    ret = this.leavePB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                if (this.matrix.pushbarea === true) {
                    ret = this.areaPB[2];
                    l1 = dSS._("Area 3");
                    showLocalPriority = true;
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                }
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                if (this.matrix.pushbarea === true) {
                    ret = this.areaPB[3];
                    l1 = dSS._("Area 4");
                    showLocalPriority = true;
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                }
                break;
            case dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL;
                    ret = this.bellPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_APARTMENT:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                l1 = this.zonePB[0][3];
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                if (l1 === undefined) {
                    l1 = this.zonePB[1][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                if (l1 === undefined) {
                    l1 = this.zonePB[2][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                if (l1 === undefined) {
                    l1 = this.zonePB[3][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                if (l1 === undefined) {
                    l1 = this.zonePB[4][3];
                }
                ret = this.zonePB;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                ret = this.highlevelPB;
                l1 = "";
                break;
            case dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_INVERTED:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_ON_RISINGEDGE:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_ON_FALLINGEDGE:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_RISINGEDGE:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_FALLINGEDGE:
            case dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_SENSOR;
                    ret = this.sensorPB;
                    l1 = "";
                }
                break;
            case dSS.config.deviceParams.PUSHBUTON_DISABLED:
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");

        }
        return [ ret, value, l1, showLocalPriority, rbval ];
    },
    updateLocalPriority: function(color) {
        if ((this.group != color) &&
            (color != dSS.config.groups.JOKER) &&
            (this.group != dSS.config.groups.JOKER) &&
            ((this.group == dSS.config.groups.SHADOW) ||
             (color == dSS.config.groups.SHADOW))) {
            this.group = color;
            if (this.updateLocalPrio()) {
                if ((this.ownerCt !== undefined) &&
                    (this.ownerCt.xtype === 'panel') &&
                    (this.ownerCt.collapsed === true)) {
                        this.lpTip.initTarget(this.ownerCt.getId());
                        this.lpTip.enable();
                        this.lpTip.show();
                }
            }
        } else {
            this.group = color;
            // Hide tooltip again
            if ((this.ownerCt !== undefined) &&
                (this.ownerCt.xtype === 'panel')) {
                    this.lpTip.disable();
                    this.lpTip.hide();
            }
        }

        // #18906 disable local prio and preset selection in joker group
        if (this.group === dSS.config.groups.JOKER) {
            this.disableElements();
            this.localPrio.disable();
        } else {
            this.sceneSelection.enable();
            this.label1.enable();
            this.label2.enable();
            this.localPrio.enable();
        }
    },
    enableElements: function() {
        this.sceneSelection.enable();
        this.label1.enable();
        this.label2.enable();
        this.callsPresent.enable();
    },
    disableElements: function() {
        this.sceneSelection.disable();
        this.label1.disable();
        this.label2.disable();
    },
    updateForm: function(value) {
        if ((value != dSS.devprops.PBConfig.PUSHBUTTON_DISABLED) &&
            (this.ltmode == dSS.config.deviceParams.BUTTONINPUT_DISABLED)) {

            this.ltmode = dSS.config.deviceParams.BUTTONINPUT_STANDARD;
        }

        if (this.currentValue === value) {
            return;
        }

        if ((value < 0) && (dSS.util.isUserGroup(this.loadedActiveGroup))) {
            this.activeGroup = dSS.config.ResetActiveButtonGroup;
        }

        var areaPushButtons = [
            dSS.devprops.PBConfig.PUSHBUTTON_AREA1,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA2,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA3,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA4
        ];
        var areaIndex = areaPushButtons.indexOf(value);
        // It may happen that dSS.devprops.PBConfig.PUSHBUTTON_AREA was selected before, 
        // and this.sceneSelection is already filled with correct data by another loader.
        // If now a specific area is selected, e.g. PUSHBUTTON_AREA1, we should not
        // overwrite existing data with default values:
        var specificAreaGotSelected = this.currentValue === dSS.devprops.PBConfig.PUSHBUTTON_AREA && areaIndex !== -1;

        this.currentValue = value;

        if (areaIndex !== -1 && this.matrix.pushbarea === true) {
            var defaultSceneSelectionItemsForArea = this.areaPB[areaIndex];
            var currentSceneSelectionItems = this.sceneSelection.getStore().getRange().map(function (item) {
                return item.json;
            });
            if (!specificAreaGotSelected || JSON.stringify(currentSceneSelectionItems) !== JSON.stringify(defaultSceneSelectionItemsForArea)) {
                this.sceneSelection.getStore().loadData(this.areaPB[areaIndex]);
                this.sceneSelection.setValue(this.areaPB[areaIndex][0][0]);
            }
            var sceneSelectionValue = this.sceneSelection.getValue();
            var selectedPreset = this.areaPB[areaIndex].find(function (item) {
                return item[0] === sceneSelectionValue;
            });
            this.label1.setText(selectedPreset[3]);     // e.g. "Area 3"
            this.label2.setText(selectedPreset[1]);     // e.g. dSS._("Scenes 32-34")
            this.enableElements();
            this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
                this.sceneSelection.getStore().loadData(this.devicePB);
                this.sceneSelection.setValue(this.devicePB[0][0]);
                this.label1.setText(dSS._("Device"));
                this.label2.setText(this.devicePB[0][1]);
                this.enableElements();
                this.callsPresent.enable();
        } else if ((value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) ||
                   (value == dSS.devprops.PBConfig.PUSHBUTTON_APARTMENT) ||
                   dSS.util.isUserGroup(value)) {
                this.sceneSelection.getStore().loadData(this.zonePB);
                this.sceneSelection.setValue(this.zonePB[0][0]);
                this.label1.setText(this.zonePB[0][3]);
                this.label2.setText(this.zonePB[0][2]);
                this.enableElements();
                this.callsPresent.enable();
                if (dSS.util.isUserGroup(value)) {
                    this.activeGroup = value;
                }
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                this.sceneSelection.getStore().loadData(this.highlevelPB);
                this.sceneSelection.setValue(this.highlevelPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_PANIC) {
                this.sceneSelection.getStore().loadData(this.panicPB);
                this.sceneSelection.setValue(this.panicPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME) {
                this.sceneSelection.getStore().loadData(this.leavePB);
                this.sceneSelection.setValue(this.leavePB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) {
                this.sceneSelection.getStore().loadData(this.bellPB);
                this.sceneSelection.setValue(this.bellPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ALARM) {
                this.sceneSelection.getStore().loadData(this.alarmPB);
                this.sceneSelection.setValue(this.alarmPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.enable();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_SENSOR) {
                this.sceneSelection.getStore().loadData(this.sensorPB);
                this.sceneSelection.setValue(this.sensorPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.disable();
                // sensor LT mode will be set via akm input event
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DISABLED) {
                this.sceneSelection.getStore().loadData([]);
                this.sceneSelection.setValue();
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
                this.callsPresent.disable();
                this.ltmode = dSS.config.deviceParams.BUTTONINPUT_DISABLED;
        } else {
                this.sceneSelection.getStore().loadData([]);
                this.sceneSelection.setValue();
                this.sceneSelection.markInvalid(
                        dSS._("Unknown pushbutton configuration!"));
                this.label1.setText("");
                this.label2.setText("");
        }
        this.updateLocalPrio(specificAreaGotSelected);
    },
    updateLocalPrio: function(specificAreaGotSelected) {
        var value = this.currentValue;
        
        var areaPushButtons = [
            dSS.devprops.PBConfig.PUSHBUTTON_AREA1,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA2,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA3,
            dSS.devprops.PBConfig.PUSHBUTTON_AREA4
        ];

        if (areaPushButtons.indexOf(value) !== -1) {
            this.localPrio.setDisabled(this.group === dSS.config.groups.JOKER);
            if (!specificAreaGotSelected || this.loadedPrioLocalScene == null) {
                if (this.group === dSS.config.groups.SHADOW) {
                    this.localPrio.setValue(0);
                } else {
                    this.localPrio.setValue(1);
                }
                return true;
            }
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
            this.localPrio.setDisabled(this.group === dSS.config.groups.JOKER);
            this.localPrio.setValue((this.loadedPrioLocalScene) ? 1 : 0);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
            this.localPrio.setDisabled(true);
            this.localPrio.setValue(0);
        } else {
            this.localPrio.setDisabled(true);
        }
        return false;
    },

    handleAdvancedView: function(group, device) {
        if (device.get('buttonInputCount') == 0) {
            return;
        }

        if (group === dSS.config.groups.TEMPCONTROL) {
            this.disable();
            this.ltmode = 65;
        } else {
            this.enable();
        }

        if ((group === dSS.config.groups.HEATING) ||
            (group === dSS.config.groups.COOLING) ||
            (group === dSS.config.groups.VENTILATION)) {
            this.ltmode = 0;
        }
    },
    saveAKMInput: function(input) {
        this.AKMInputValue = dSS.config.ButtonInputModeLookup[input];
    },
    updateVisibility: function(input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.disable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.enable();
        }
    }
});

Ext.reg('dssdevgrouppushbuttonadvancedform', dSS.devprops.GroupPushbuttonAdvancedForm);

