Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured group)
dSS.devprops.HeatingConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.device = config.device;

        var fid = dSS.util.parseFunctionID(this.device.get('functionID'))
        var devClass = fid[0];
        var productID = this.device.get('productID');
        var pid = dSS.util.parseProductID(productID);
        var devType = dSS.config.deviceTypes[pid[0]];
        var devNumber = pid[1];

        var list =
        [
            [
                dSS.config.groups.HEATING,
                dSS._("Heating manual")
            ],
            [
                dSS.config.groups.TEMPCONTROL,
                dSS._("Heating/Cooling regulated")
            ]
        ];

        if ((devClass === dSS.config.deviceClass.BL) &&
            (devType === "KM") &&
            (devNumber === 300)) {
            list = list.concat([
                [
                    dSS.config.groups.VENTILATION,
                    dSS._("Room ventilation")
                ],
                [
                    dSS.config.groups.RECIRCULATION,
                    dSS._("Air recirculation")
                ],
                [
                    dSS.config.groups.APT_VENTILATION,
                    dSS._("Apartment ventilation")
                ]
            ]);
        }

        this.dSUID = config.dSUID;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded');

        this.groupSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 175,
            mode: 'local',
            fieldLabel: dSS._("Application"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            height: 35,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.groupSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.HeatingConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.groupSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.groupSelection.on('select', function(box, record, index) {
            dSS.MessageBus.fireEvent('heatinggroupformchanged', record.data.id, this.device);
            this.fireEvent('formchanged');
        },
        this);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);

        this.groupSelection.on('afterrender', function() {
                if (this.closed !== true) {
                    this.groupSelection.setValue(this.group);
                    dSS.MessageBus.fireEvent('heatinggroupformchanged',
                                             this.group, this.device);
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                var value = this.groupSelection.getValue();
                var dSUID = this.dSUID;
                var group = this.group;
                if (value == group) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                Ext.Ajax.request({
                    url: '/json/device/setHeatingGroup',
                    disableCaching: true,
                    method: "GET",
                    params:
                    {
                        dsuid : this.dSUID,
                        groupID : value
                    },
                    scope: this,
                    success: function(result, request) {
                        try {
                            var json = Ext.util.JSON.decode(result.responseText);
                            if (!json.ok) {
                                Ext.Msg.alert(dSS._("Error"),
                                    dSS._("Could not set device group") +
                                    ': (' + json.message + ')');
                                this.fireEvent('formsubmitted');
                                return;
                            }
                            dSS.util.processDeviceActionResponse(json.result);
                        } catch (err) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not set device group") +
                                ': (' + err + ')');
                        }
                        this.fireEvent('formsubmitted');
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set device group"));
                        this.fireEvent('formsubmitted');
                    }
                });
        },
        this);
    }
});

Ext.reg('dssdevheatingconfigform', dSS.devprops.HeatingConfigForm);

