Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured group)
dSS.devprops.ValveTypeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.device === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing device"));
            return;
        }

        var list =
        [
            [
                dSS.config.ValveTypes.UNKNOWN,
                dSS._("Unknown")
            ],
            [
                dSS.config.ValveTypes.FLOOR,
                dSS._("Floor heating")
            ],
            [
                dSS.config.ValveTypes.WALL,
                dSS._("Wall")
            ],
            [
                dSS.config.ValveTypes.RADIATOR,
                dSS._("Radiator")
            ]
        ];

        if (config.matrix.extendedvalvetypes === true) {
            list = list.concat([
                [
                    dSS.config.ValveTypes.CONVECTOR_PASSIVE,
                    dSS._("Convector passive")
                ],
                [
                    dSS.config.ValveTypes.CONVECTOR_ACTIVE,
                    dSS._("Convector active")
                ],
                [
                    dSS.config.ValveTypes.FLOOR_LOW_ENERGY,
                    dSS._("Floor heating low energy")
                ]
            ]);
        }

        this.device = config.device;
        this.dSUID = this.device.get('dsuid');
        this.valveType = dSS.config.ValveTypes.UNKNOWN;

        var disabled = false;
        var group = dSS.util.getJokerGroup(this.device.get('groups'));
        if ((group != dSS.config.groups.HEATING) &&
            (group != dSS.config.groups.TEMPCONTROL)) {
            disabled = true;
        }

        this.addEvents('formsubmitted', 'formloaded');

        this.valveSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            fieldLabel: dSS._("Attached native device"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            disabled: disabled,
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.valveSelection
                    ]
                }
            ]
        });

        dSS.devprops.ValveTypeForm.superclass.constructor.apply(this, arguments);

        dSS.MessageBus.on('jokerconfigformchanged', this.updateVisibility, this);

        dSS.MessageBus.on('heatinggroupformchanged', this.updateVisibility, this);
        this.on('loadcomplete', function() {
                this.valveSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
                dSS.MessageBus.removeListener('jokerconfigformchanged',
                                              this.updateVisibility, this);

                dSS.MessageBus.removeListener('heatinggroupformchanged',
                                              this.updateVisibility, this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.VALVE_TYPE) {
                    return;
                }

                if (data.json.ok === true) {
                    this.valveType = data.json.result.valveType;
                    this.valveSelection.setValue(this.valveType);
                } else {
                    this.valveSelection.markInvalid(data.json.message);
                }


                this.fireEvent('formloaded', true);
            },
            this);


        this.on('submit', function() {
                if (this.valveSelection.disabled) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                var value = this.valveSelection.getValue();
                var dSUID = this.dSUID;

                if (this.valveType == value) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                dSS.util.genericRequest({
                    url: '/json/device/setValveType',
                    scope: this,
                    params:
                    {
                        dsuid: this.dSUID,
                        valveType: value
                    },
                    failureMsg: dSS._("Could not set device valve type"),
                    evt: 'formsubmitted'
                });
            },
            this);
    },
    updateVisibility: function(group) {
        if ((group == dSS.config.groups.HEATING) ||
            (group == dSS.config.groups.TEMPCONTROL)) {
            this.valveSelection.enable();
        } else {
            this.valveSelection.disable();
        }
    }
});

Ext.reg('dssdevvalvetypeform', dSS.devprops.ValveTypeForm);

