//= require <dSS/devices/device-settings/device-type-panels/DeviceSettingsHSVView>
//= require <dSS/devices/device-settings/device-type-panels/DeviceSettingsHSVViewApartmentMode>

Ext.namespace("dSS", "dSS.devices", "dSS.config", "dSS.gthelper");

// devClass
//  stop button will be shown depending on the device class
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');
        this.notFirstCall = false;
        this.dSUID = config.dSUID;
        this.scene = config.scene;
        this.dummy = false;
        if (config.dummy !== undefined) {
            this.dummy = config.dummy;
        }

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.matrix = config.matrix;
        this.virtualDevice = config.virtualDevice;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 210;
        this.EXTRA_HEIGHT_GRKL387HACK = this.EXTRA_HEIGHT + 25;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS = 332;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS_TEMPERATURE_ONLY = 240;
        this.EXTRA_HEIGHT_FCU = 246;
        var initialHeight = this.DEFAULT_HEIGHT;
        if (this.matrix.fcu === true) {
            initialHeight = this.EXTRA_HEIGHT_FCU;
        } else if (this.matrix.shadebladeang === true) {
            initialHeight = this.EXTRA_HEIGHT;
        } else if (this.matrix.outputchannels === true) {
            initialHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
        } else if ((this.apartmentMode === true) &&
                   (this.matrix.grkl387workaround === true)) {
            initialHeight = this.EXTRA_HEIGHT_GRKL387HACK;
        }

        if (config.overrideHeight !== undefined) {
            this.EXTRA_HEIGHT = config.overrideHeight;
            initialHeight = config.overrideHeight;
        }


        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        var xPosOffset = 280;

        this.noconfig = new Ext.Panel({
            x: 0,
            y: 0,
            border: false
        });

        var colorPickerConfig = {
            x: 0,
            y: 0,
            dSUID: this.dSUID
        };
        this.colorPicker = this.apartmentMode
            ? new dSS.devices.DeviceSettingsHSVViewApartmentMode(colorPickerConfig)
            : new dSS.devices.DeviceSettingsHSVView(colorPickerConfig);

        this.videoView = new dSS.devices.DeviceSettingsVideoView({
            x: 0,
            y: 0,
            initialHeight: initialHeight,
            dSUID: this.dSUID,
            apartmentMode: this.apartmentMode,
            device: config.device
        });

        this.configLink = new dSS.devices.DeviceSettingsVdcConfigLinkView({
            x: 0,
            y: 0,
            initialHeight: this.DEFAULT_HEIGHT,
            dSUID: this.dSUID,
            device: config.device
        });

        this.fcu = new dSS.devices.DeviceSettingsFCUView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: this.EXTRA_HEIGHT_FCU,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
            device: config.device
        });

        this.sliderView = new dSS.devices.DeviceSettingsDefaultView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: initialHeight,
            matrix: this.matrix,
            maxValue: this.maxValue,
            decPrecision: decPrecision,
            devClass: config.devClass,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
            device: config.device
        });

        Ext.apply(this, {
            layout: 'card',
            activeItem: 0,
            padding: 0,
            bubbleEvents: [ 'updateshadow', 'valuechanged', 'enableme',
                            'disableme', 'formchanged' ],
            height: initialHeight,
            items:
            [
                // do not reorder!
                this.noconfig,
                this.sliderView,
                this.colorPicker,
                this.fcu,
                this.configLink,
                this.videoView
            ]
        });

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.addEvents('updateshadow');

        this.on('afterrender', function() {
                this.fireEvent('updateshadow', initialHeight);
                this.setData(this.dSUID, config.devClass, config.present,
                     config.matrix, config.virtualDevice, config.device);
            },
            this);

        this.on('beforedestroy', function() {
                this.killRequest();
                this.closed = true;
            },
            this);

        this.on('enableme', function() {
            if (this.apartmentMode !== true) {
                this.enable();
            }
        },
        this);
        this.on('disableme', function() {
            if (this.apartmentMode !== true) {
                this.disable();
            }
        },
        this);

    },
    visualConfig: function(matrix, devClass, virtualDevice) {
        if (this.apartmentMode !== true) {
            this.fireEvent('disableme');
        }

        if (matrix.customactivityconfig) {
            this.getLayout().setActiveItem(0);
        } else if (devClass === dSS.config.deviceClass.MG && virtualDevice === true) {
            this.getLayout().setActiveItem(5); // video view
        } else if ((matrix.outvalue8 || matrix.shadeposition) &&
            (!matrix.outputchannels) && (!matrix.fcu)) {
            this.getLayout().setActiveItem(1); // default slider view
        } else if (matrix.outputchannels) {
            this.getLayout().setActiveItem(2); // color picker
	    } else if (matrix.fcu) {
            this.getLayout().setActiveItem(3); // fcu view
        } else if (virtualDevice === true) {
            this.getLayout().setActiveItem(4); // vdc configuration link
        } else {
            this.getLayout().setActiveItem(0);
        }

        this.doLayout(false, true);
    },

    setData: function(dSUID, devClass, present, matrix, virtualDevice, record) {
        this.dSUID = dSUID;
        this.matrix = matrix;

        if ((this.apartmentMode === true) &&
            (matrix.grkl387workaround === true)) {
            if (this.getHeight() !== this.EXTRA_HEIGHT_GRKL387HACK) {
                this.setHeight(this.EXTRA_HEIGHT_GRKL387HACK);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_GRKL387HACK);
            }
        } else if (matrix.shadebladeang === true) {
            if (this.getHeight() !== this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        } else if (matrix.outputchannels === true) {
            var supportsHSVChannels = record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'hue' || channel.channelId === 'saturation';
            });
            var supportsTemperatureChannel = record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'colortemp';
            });
            var newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
            if (!supportsHSVChannels) {
                newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS_TEMPERATURE_ONLY - 15;    // there'll be no "Switch to other view" label
            } else if (!supportsTemperatureChannel) {
                newHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS - 15;     // there'll be no "Switch to other view" label
            }
            if (this.getHeight() !== newHeight) {
                this.setHeight(newHeight);
                this.fireEvent('updateshadow', newHeight);
            }
        } else if (matrix.fcu === true) {
            if (this.getHeight() !== this.EXTRA_HEIGHT_FCU) {
                this.setHeight(this.EXTRA_HEIGHT_FCU);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_FCU);
            }
        // } else if (virtualDevice === true) {
        //     if (this.getHeight() !== this.EXTRA_HEIGHT + 15) {
        //         this.setHeight(this.EXTRA_HEIGHT);
        //         this.fireEvent('updateshadow', this.EXTRA_HEIGHT + 15);
        //     }
        } else {
            if (this.getHeight() !== this.DEFAULT_HEIGHT) {
                this.setHeight(this.DEFAULT_HEIGHT);
                this.fireEvent('updateshadow', this.DEFAULT_HEIGHT);
            }
        }

        this.clearData();

        var bl361_hack = false;
        if (devClass === dSS.config.deviceClass.BL) {
            var devInfo = dSS.util.parseProductID(record.get('productID'));
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if (((devType === "KM") && (devInfo[1] === 200)) &&
                    (matrix.pwmvalue === true) && (record.get('productRevision') == 0x361) &&
                    (record.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {
                bl361_hack = true;
            }
        }

        this.sliderView.visualConfig(matrix, devClass, virtualDevice, bl361_hack);
        this.visualConfig(matrix, devClass, virtualDevice);

        if (!present) {
            return;
        }

        if ((devClass === dSS.config.deviceClass.MG) && (virtualDevice === true)) {
            this.videoView.loadData(dSUID, record, (this.notFirstCall != true));
        } else if ((matrix.outvalue8 || matrix.shadeposition) &&
            (!matrix.outputchannels) && (!matrix.fcu)) {
            this.sliderView.loadData(dSUID, devClass, matrix, virtualDevice,
                                     bl361_hack, record);
        } else if (matrix.outputchannels) {
            this.colorPicker.loadData(dSUID, record);
	    } else if (matrix.fcu) {
            this.fcu.loadData(dSUID);
        } else if (virtualDevice === true) {
            this.configLink.loadData(dSUID, record, (this.notFirstCall != true));
        }

        this.doLayout(false, true);
        this.notFirstCall = true;
    },
    killRequest: function() {
        this.sliderView.killRequests();
        this.colorPicker.killRequests();
        this.videoView.killRequests();
        this.configLink.killRequests();
        this.fcu.killRequests();
    },
    clearData: function() {
        this.sliderView.clearData();
        this.colorPicker.clearData();
        this.fcu.clearData();
    },
    doPreview: function() {
        this.colorPicker.doPreview();
        this.sliderView.doPreview();
        this.fcu.doPreview();
        this.videoView.doPreview();
    },
    saveSceneValues: function() {
        if (this.apartmentMode !== true) {
            return;
        }
        this.colorPicker.saveSceneValues(this.scene);
        this.sliderView.saveSceneValues(this.scene);
        this.fcu.saveSceneValues(this.scene);
        this.videoView.saveSceneValues(this.scene);
	},
    resetDefaults: function() {
        this.sliderView.resetDefaults();
        this.colorPicker.resetDefaults();
        this.fcu.resetDefaults();
        this.videoView.resetDefaults();
    },
    resetValues: function() {
        this.sliderView.resetValues();
        this.colorPicker.resetValues();
        this.fcu.resetValues();
        this.videoView.resetValues();
    }

});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);
