Ext.namespace("dSS", "dSS.devices");

// *STANDARD
dSS.devices.DeviceSettingsDefaultView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.maxValue = config.maxValue;
        this.closed = false;
        this.dSUID = config.dSUID;
        this.device = config.device;

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.dirtyValue = false;
        this.dirtyAngle = false;

        this.previousValue = 0;
        this.previousBladeAngle = 0;
        this.decPrecision = (config.decPrecision === 0) ? 1 : 0.001;

        var increment = this.decPrecision;
        if ((this.apartmentMode === true) &&
            (config.matrix.grkl387workaround === true)) {
            increment = dSS.config.GRKL387SliderIncrement;
        }

        var radioItems =
        [
            {
                boxLabel: dSS._("Normal mode"),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.STOP
            },
            {
                boxLabel: dSS._("Force to 100%"),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.LOCAL_ON
            },
            {
                boxLabel: dSS._("Force to 0%"),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.LOCAL_OFF
            }

        ];

        var bl361_hack = false;
        if (config.devClass === dSS.config.deviceClass.BL) {
            var devInfo = dSS.util.parseProductID(config.device.get('productID'));
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if (((devType === "KM") && (devInfo[1] === 200)) &&
                    (config.matrix.pwmvalue === true) && (config.device.get('productRevision') == 0x361) &&
                    (config.device.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {
                bl361_hack = true;
            }
        }

        Ext.apply(this, {
            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme' ],
            items:
            [
                {
                    xtype: 'label',
                    ref: 'valueLabel',
                    x: 10,
                    y: 16,
                    disabled: this.dummy,
                    hidden: bl361_hack,
                    text: (config.devClass === dSS.config.deviceClass.BL) ? dSS._("Flow rate") + ':' : dSS._("Value") + ':'
                },
                {
                    xtype: 'numberfield',
                    ref: 'valueField',
                    x: xPos - xPosOffset,
                    y: 12,
                    width: 50,
                    allowBlank: false,
                    hidden: bl361_hack,
                    allowNegative: false,
                    allowDecimals: (config.decPrecision !== 0),
                    decimalPrecision: config.decPrecision,
                    forceDecimalPrecision: true,
                    disabled: (config.devClass === dSS.config.deviceClass.BL),
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    x: xPos - (xPosOffset - 55),
                    y: 16,
                    ref: 'valuePostLabel',
                    hidden: bl361_hack,
                    xtype: 'label',
                    text: '%'
                },
                {
                    xtype: 'slider',
                    ref: 'valueSlider',
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    height: 150,
					value: 0,
                    minValue: 0,
                    maxValue: 100,
                    vertical: true,
                    increment: increment,
                    decimalPrecision: config.decPrecision,
                    hidden: (config.devClass === dSS.config.deviceClass.BL),
                    plugins: new Ext.slider.Tip({
                        getText: function(thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'button',
                    ref: 'stopButton',
                    x: xPos - xPosOffset,
                    y: 140,
                    text: dSS._("Stop"),
                    width: 50,
                    handler: this.handleStop.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'maxButton',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    text: dSS._("Max"),
                    tooltip: dSS._("Set maximum value"),
                    width: 130,
                    handler: this.handleMax.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'minButton',
                    x: xPos - (xPosOffset - 102),
                    y: 140,
                    text: dSS._("Min"),
                    tooltip: dSS._("Set minimum value"),
                    width: 130,
                    handler: this.handleMin.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'incButton',
                    x: xPos - (xPosOffset - 102),
                    y: 54,
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 130,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'decButton',
                    x: xPos - (xPosOffset - 102),
                    y: 98,
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 130,
                    handler: this.handleDec.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'flowRateLabel',
                    x: 10,
                    y: 43,
                    text: dSS._("Control of flow rate") + ':',
                    hidden: (config.devClass !== dSS.config.deviceClass.BL)
                },
                {
                    xtype: 'radiogroup',
                    ref: 'flowRateSelection',
                    columns: 1,
                    items: radioItems,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL),
                    x: xPos - xPosOffset - 5,
                    y: 41
                },
                {
                    xtype: 'label',
                    ref: 'forceModeHelp',
                    hidden: (config.devClass !== dSS.config.deviceClass.BL),
                    text: dSS._("Forced flow rate resets to normal mode after 6 hours."),
                    x: xPos - xPosOffset,
                    y: 115
                },
                {
                    xtype: 'label',
                    ref: 'opModeLabel',
                    x: 10,
                    y: 152,
                    text: dSS._("Current operation mode") + ':',
                    hidden: (config.devClass !== dSS.config.deviceClass.BL)
                },
                {
                    xtype: 'label',
                    ref: 'opModeText',
                    x: xPos - xPosOffset,
                    y: 152,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL)
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleLabel',
                    x: 10,
                    y: 178,
                    text: dSS._("Set blade angle") + ':'
                },
                {
                    xtype: 'numberfield',
                    ref: 'bladeAngleField',
                    x: xPos - xPosOffset,
                    y: 174,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleUnit',
                    x: xPos - (xPosOffset - 55),
                    y: 178,
                    text: '%'
                },
                {
                    xtype: 'slider',
                    ref: 'angleSlider',
                    x: xPos - (xPosOffset - 74),
                    y: 174,
                    width: 160,
		            value: 0, 
                    increment: 1,
                    minValue: 0,
                    maxValue: 100,
                    vertical: false,
                    decimalPrecision: 1,
                    plugins: new Ext.slider.Tip({
                        getText: function(thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'grKLHack',
                    x: 10,
                    y: 205,
                    text: dSS._("This device only supports output values in 6.66% steps."),
                    hidden: !((this.apartmentMode === true) && (config.matrix.grkl387workaround === true))
                }
            ]
        });

        dSS.devices.DeviceSettingsDefaultView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.flowRateSelection.on('change', function(radiogroup, radio) {
            if (this.apartmentMode === true) {
                return;
            }
            dSS.util.callScene(this.dSUID, radio.getRawValue());
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        },
        this);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = parseFloat(field.getValue());
                    var ov = parseFloat(this.valueSlider.getValue());
                    if ((this.apartmentMode === true) &&
                        (this.matrix.grkl387workaround === true)) {
                        nv = (Math.round(nv*15/100)/15)*100;
                        field.setValue(nv);
                    };
                    if (nv !== ov) {
                        this.handleValueChanged(field, nv);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.handleBladeAngleChanged(field, nv);
                }
            },
            this);
    },
    setMatrix: function(matrix) {
        this.matrix = matrix;
    },
    handleValueChanged: function(container, newVal, oldVal) {
        if (newVal === '') {
            this.valueField.setValue(this.valueSlider.getValue());
            return;
        }
        var value = dSS.devices.helpers.ensureValueWithinRange({
            value: newVal,
            min: 0,
            max: 100
        });
        if (value === oldVal) {
            if (value !== newVal) {
                this.valueField.setValue(value);
            }
            return;
        }

        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.valueSlider.setValue(value);
        this.valueField.setValue(value);
        this.dirtyValue = true;
        if (this.apartmentMode !== true) {
            this.setValueOnDevice(value);
        }
        if (this.matrix.shadebladeang === true) {
            if (value >= 90) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 100 ]);
            }
        }
    },
    setValueOnDevice: function(value) {
        if (value === '') {
            return;
        }
        this.previousValue = value;

        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dSUID,
                                      dSS.util.percentToValue(value,
                                                    this.maxValue));
        } else {
            dSS.util.setOutputValueOnDevice(this.dSUID, 0x02,
                                        dSS.util.percentToValue(value,
                                                    this.maxValue));
        }
    },
    handleBladeAngleChanged: function(container, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.dirtyAngle = true;
        this.bladeAngleField.setValue(newVal);
        this.angleSlider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (newVal !== this.previousBladeAngle)) {
            this.setAngleOnDevice(newVal);
        }
        this.previousBladeAngle = newVal;
    },
    setAngleOnDevice: function(angle) {
        dSS.util.genericRequest({
                url: '/json/device/setConfig',
                scope: this,
                params:
                {
                    'class': 64,
                    index: 0x04,
                    dsuid: this.dSUID,
                    value: Math.round(angle / 100 * 0xff)
                },
                failureMsg: dSS._("Could not set device blade angle")
            });
    },
    handleMax: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.MAX);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(100);
            this.angleSlider.setValue(100);
            this.dirtyAngle = false;
            if (this.apartmentMode !== true) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 100 ]);
            }
        }
    },
    handleMin: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.MIN);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(0);
            this.angleSlider.setValue(0);
            this.dirtyAngle = false;
            if (this.apartmentMode !== true) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 0 ]);
            }
        }
    },
    handleStop: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
        } else {
            this.fireEvent('formchanged');
        }
        if ((this.apartmentMode !== true) &&
            (this.matrix.shadeposition === true)) {
            this.dirtyValue = true;
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        }
    },
    handleInc: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }
            
        this.dirtyValue = true;

        if ((this.matrix.shadeposition === true) && 
            (this.apartmentMode !== true)) {

            dSS.util.callScene(this.dSUID, dSS.config.scenes.INC);
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
            return;
        }

        var v = this.valueSlider.getValue() + 5;
        if (v > 100) {
            v = 100;
        }

        if (v != this.valueSlider.getValue()) {
            this.valueSlider.setValue(v);
            this.handleValueChanged(this.valueSlider, v);
        }
    },
    handleDec: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }

        this.dirtyValue = true;

        if ((this.matrix.shadeposition === true) &&
            (this.apartmentMode !== true)) {

            dSS.util.callScene(this.dSUID, dSS.config.scenes.DEC);
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
            return;
        }

        var v = this.valueSlider.getValue() - 5;
        if (v < 0) {
            v = 0;
        }

        this.handleValueChanged(this.valueSlider, v);
    },
    clearData: function()
    {
        this.killRequests();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.previousValue = 0;
        this.previousBladeAngle = 0;
    },
    killRequests: function() {
		this.fireEvent('loading', false);
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
        }

        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }

        if (this.AjaxRequestOperationMode !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOperationMode);
            this.AjaxRequestOperationMode = undefined;
        }
    },
    readBladeAngleValue: function(reqdSUID) {
        this.killRequests();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var offset = 0x04;

        this.dirtyAngle = false;
        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);
                        this.bladeAngleField.markInvalid(
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);

                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * 100 / 0xff);
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                        this.angleSlider.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        jsonData.message);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + jsonData.message);

                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            }
        });
    },
    readOperationMode: function(reqdSUID) {
        this.killRequests();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        var bl361_hack = false;
        // yet another BL-KM200 hack for firmware 3.6.1
        var devCls = dSS.util.parseFunctionID(this.device.get('functionID'));
        var devClass = devCls[0];
        if (devClass === dSS.config.deviceClass.BL) {
            var devInfo = dSS.util.parseProductID(this.device.get('productID'));
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if (((devType === "KM") && (devInfo[1] === 200)) &&
                    (this.matrix.pwmvalue === true) && (this.device.get('productRevision') == 0x361)) {
                bl361_hack = true;
            }
        }

        this.fireEvent('loading', true);
        this.opModeText.setText(dSS._("loading") + '...');
        var request = 'getConfig';
        var offset = 0x05;

        this.AjaxRequestOperationMode = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device operation mode") +
                            ': ' + jsonData.message);
                        this.opModeText.setText("");
                        this.flowRateSelection.suspendEvents(false);
                        this.flowRateSelection.setValue("");
                        this.resumeFlowRateEvents.defer(800, this, [ this ]);

                    } else if (!isNaN(jsonData.result.value)) {
                        var text = dSS.config.HeatingOperationModeStrings[jsonData.result.value];
                        this.opModeText.setText(dSS._(text));
                        // #dsd-484:
                        // modes 0-3 => normal
                        // mode 4    => forced on
                        // mode 5    => forced off
                        this.flowRateSelection.suspendEvents(false);
                        switch (jsonData.result.value) {
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                                this.flowRateSelection.setValue(dSS.config.scenes.STOP);
                                break;
                            case 4:
                                this.flowRateSelection.setValue(dSS.config.scenes.LOCAL_ON);
                                if (bl361_hack === true) {
                                    this.valueSlider.setValue(100);
                                    this.valueField.setValue(100);
                                    this.previousValue = 100;
                                }
                                break;
                            case 5:
                                this.flowRateSelection.setValue(dSS.config.scenes.LOCAL_OFF);
                                if (bl361_hack === true) {
                                    this.valueSlider.setValue(0);
                                    this.valueField.setValue(0);
                                    this.previousValue = 0;
                                }
                                break;
                            default:
                                break;
                        }
                        // hack: deferring flowRateSelection.resumeEvents()
                        // directly did not work
                        this.resumeFlowRateEvents.defer(800, this, [ this ]);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue("");
                    this.resumeFlowRateEvents.defer(800, this, [ this ]);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        jsonData.message);
                    this.opModeText.setText("");
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue("");
                    this.resumeFlowRateEvents.defer(800, this, [ this ]);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue("");
                    this.resumeFlowRateEvents.defer(800, this, [ this ]);
                }
            }
        });
    },
    resumeFlowRateEvents: function(thiz) {
        if (thiz.flowRateSelection !== undefined) {
            thiz.flowRateSelection.resumeEvents();
        }
    },
    loadData: function(reqdSUID, devClass, matrix, virtualDevice, bl361_hack,
                       device)
    {
        this.killRequests();

        this.dSUID = reqdSUID;
        this.device = device;

        this.matrix = matrix;

        this.fireEvent('loading', true);

        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);

        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            return;
        }

        // #dsd-484: broken 3.6.1 firmware hack for BL-KM
        if (bl361_hack === true) {
            this.fireEvent('loading', false);
            return; // dialog won't get enabled
        }

        var devInfo = dSS.util.parseProductID(device.get('productID'));
        // output mode dependency, hack for BL-KM
        if (devClass === dSS.config.deviceClass.BL) {
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if ((devType === "KM") && ((devInfo[1] === 200) || (devInfo[1] === 300))) {
                var hack = dSS.util.getHeatingMods(matrix,
                                                   device.get('outputMode'));
                matrix = hack.matrix;
                devClass = hack.deviceClass;
            }
        }

        this.fireEvent('enableme');

        this.visualConfig(matrix, devClass, virtualDevice, false);

        this.readDeviceValue(reqdSUID);
    },
    readDeviceValue: function(dSUID) {
        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= 0xff) {
            request = 'getConfig';
            offset = 0;
        }

        // when in modes 64 and 65 read from bank 64, offset 4
        if (this.matrix.pwmvalue === true) {
            if ((this.device.get('outputMode') ==
                    dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM) ||
                (this.device.get('outputMode') ==
                    dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {
                request = 'getConfig';
                offset = 4;
            }
        }

        this.dirtyValue = false;
        this.dirtyAngle = false;

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: dSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if ((this.closed === true) ||
                    (this.dirtyValue === true)  ||
                    (dSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                        this.valueField.markInvalid(
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var p;
                        if (this.matrix.pwmvalue === true) {
                            p = jsonData.result.value;
                        } else {
                            p = dSS.util.valueToPercent(jsonData.result.value,
                                                        this.maxValue);
                        }
                        this.valueSlider.setValue(p);
                        this.valueField.setValue(p);
                        this.previousValue = p;
                    }

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(dSUID);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(dSUID);
                    }
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            }
        });
    },
    setDeviceValue: function(value)
    {
        if (isNaN(value)) {
            return;
        }

        var p = dSS.util.valueToPercent(value, this.maxValue);
        this.valueSlider.setValue(p);
        this.valueField.setValue(p);
    },
    setBladeAngle: function(angle) {
        if (isNaN(angle)) {
            return;
        }

        this.previousBladeAngle = angle;

        var p = Math.round(dSS.util.valueToPercent(angle, 0xff));
        this.bladeAngleField.setValue(p);
        this.angleSlider.setValue(p);
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }
        if (this.valueField.getValue() === '' || this.matrix.shadebladeang === true && this.bladeAngleField.getValue() === '') {
            return;
        }
        if (this.apartmentMode !== true) {
            return;
        }

        var value = dSS.util.percentToValue(this.valueField.getValue(), this.maxValue);
        var angle = this.matrix.shadebladeang === true
            ? dSS.util.percentToValue(this.bladeAngleField.getValue(), 0xff)
            : undefined;

        dSS.util.setSceneValue.defer(500, this,
                [this.dSUID, scene, value, angle ]);
        this.resetDefaults();
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }
        this.previousValue = this.valueField.getValue();
        if (this.matrix.shadebladeang) {
            this.previousBladeAngle = this.bladeAngleField.getValue();
        }
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        var p = this.previousValue;
        this.valueField.setValue(p);
        this.valueSlider.setValue(p);

        if (this.matrix.shadebladeang) {
            var p = this.previousBladeAngle;
            this.bladeAngleField.setValue(p);
            this.bladeAngleSlider.setValue(p);
        }
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }

        var value = this.valueField.getValue();
        if (value === '') {
            this.valueField.setValue(this.valueSlider.getValue());
            return;
        }

        this.setValueOnDevice(value);

        if (this.matrix.shadebladeang === true) {
            var angle = this.bladeAngleField.getValue();
            this.setAngleOnDevice(angle);
        }
    },
    visualConfig: function(matrix, devClass, virtualDevice, bl361_hack) {
        this.matrix = matrix;

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (devClass === dSS.config.deviceClass.BL) {
            this.valueLabel.setText(dSS._("Flow rate") + ':');
        } else {
            if (matrix.shadeposition === true) {
                this.valueLabel.setText(dSS._("Position") + ':');
            } else {
                this.valueLabel.setText(dSS._("Value") + ':');
            }
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.angleSlider.setVisible(matrix.shadebladeang);
        this.grKLHack.setVisible((this.apartmentMode === true) &&
                                 (matrix.grkl387workaround === true));

        if ((devClass === dSS.config.deviceClass.GR) ||
            (devClass === dSS.config.deviceClass.GN) ||
            (devClass === dSS.config.deviceClass.RT)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

        this.valueField.show();
        this.valueLabel.show();
        this.valuePostLabel.show();

		if (devClass === dSS.config.deviceClass.GR) {
            this.minButton.setText(dSS._("Down"));
            this.maxButton.setText(dSS._("Up"));
            this.flowRateLabel.hide();
            this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.show();
            if ((this.apartmentMode === true) &&
                (matrix.grkl387workaround === true)) {
                this.valueSlider.increment = dSS.config.GRKL387SliderIncrement;
                this.valueSlider.keyIncrement = dSS.config.GRKL387SliderIncrement;
            } else {
                this.valueSlider.increment = this.decPrecision;
                this.valueSlider.keyIncrement = this.decPrecision;
            }
            this.valueSlider.enable();
        } else if (devClass === dSS.config.deviceClass.BL) {
            this.maxButton.hide();
            this.minButton.hide();
            this.decButton.hide();
            this.incButton.hide();
            if (virtualDevice === true) {
                this.flowRateLabel.hide();
                this.flowRateSelection.hide();
                this.forceModeHelp.hide();
                this.opModeLabel.hide();
                this.opModeText.hide();
            } else {
                this.flowRateLabel.show();
                this.flowRateSelection.show();
                this.forceModeHelp.show();
                this.opModeLabel.show();
                this.opModeText.show();
                this.opModeText.setText(dSS._("loading") + '...');
            }
            this.valueField.disable();
            this.valueSlider.hide();
            if (bl361_hack === true) {
                this.valueField.hide();
                this.valueLabel.hide();
                this.valuePostLabel.hide();
            }

        } else {
            this.minButton.setText(dSS._("Min"));
            this.maxButton.setText(dSS._("Max"));
            this.flowRateLabel.hide();
            this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.show();
        }
    }
});