//= require <dSS/devices/device-settings/device-type-panels/color-view/DeviceSettingsColorTemperatureView>
//= require <dSS/devices/device-settings/device-type-panels/color-view/DeviceSettingsColorView>

Ext.namespace("dSS", "dSS.devices");

// *COLOR
dSS.devices.DeviceSettingsHSVView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.dSUID = config.dSUID;

        this.defaultColor = {
            colorHSVPicker: { h: 0, s: 0, v: 0 },
            colorTemperaturePicker: { brightness: 0, temperature: 3500 }
        };

        var me = this;

        var colorViewConfig = {
            ref: 'colorHSVPicker',
            x: 0,
            y: 0,
            xPos: 450,
            xPosOffset: 280,
            height: 330,
            dSUID: this.dSUID,
            toggleView: function () {
                me.colorTemperaturePicker.show();
                me.colorHSVPicker.hide();
                me.loadData(me.dSUID, me.record);
            },
            hidden: true
        };
        var temperatureViewConfig = {
            ref: 'colorTemperaturePicker',
            x: 0,
            y: 0,
            xPos: 450,
            xPosOffset: 280,
            height: 330,
            dSUID: this.dSUID,
            toggleView: function () {
                me.colorHSVPicker.show();
                me.colorTemperaturePicker.hide();
                me.loadData(me.dSUID, me.record);
            },
            hidden: true
        }
        var panelItems = [
            new dSS.devices.DeviceSettingsColorView(colorViewConfig),
            new dSS.devices.DeviceSettingsColorTemperatureView(temperatureViewConfig)
        ];
        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme' ],
            items: panelItems
        });

        dSS.devices.DeviceSettingsHSVView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);
    },
    visiblePicker: function () {
        if (!this.colorHSVPicker.hidden) {
            return this.colorHSVPicker;
        } else if (!this.colorTemperaturePicker.hidden) {
            return this.colorTemperaturePicker;
        }
    },
    clearData: function() {
        this.colorHSVPicker.killRequests();
        this.colorHSVPicker.setColor(this.defaultColor.colorHSVPicker);
        this.colorTemperaturePicker.killRequests();
        this.colorTemperaturePicker.setColor(this.defaultColor.colorTemperaturePicker);
    },
    killRequests: function() {
        this.dirtyColor = true;
        this.visiblePicker() && this.visiblePicker().killRequests();
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().resetDefaults();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().resetValues();
    },
    loadData: function(reqdSUID, record) {
        if (this.colorHSVPicker.hidden && this.colorTemperaturePicker.hidden || this.dSUID !== reqdSUID) {
            if (record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'hue' || channel.channelId === 'saturation';
            })) {
                this.colorHSVPicker.show();
                this.colorTemperaturePicker.hide();
            } else if (record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'colortemp';
            })) {
                this.colorTemperaturePicker.show();
                this.colorHSVPicker.hide();
            }
        }
        this.record = record;
        this.dSUID = reqdSUID;

        this.visiblePicker().loadData(reqdSUID, record.json.outputChannels);
    },
    setColor: function(color) {
        if (color.hue != null && color.saturation) {
            this.colorHSVPicker.setColor({ h: color.hue, s: color.saturation, v: color.brightness });
            if (color.colortemp != null) {
                this.colorHSVPicker.switchToTheOtherColorPicker.show();
            } else {
                this.colorHSVPicker.switchToTheOtherColorPicker.hide();
            }
        } else if (color.colortemp != null) {
            this.colorTemperaturePicker.setColor({ brightness: color.brightness, temperature: color.colortemp });
            if (color.hue != null && color.saturation != null) {
                this.colorTemperaturePicker.switchToTheOtherColorPicker.show();
            } else {
                this.colorTemperaturePicker.switchToTheOtherColorPicker.hide();
            }
        }
    }
});