Ext.namespace("dSS", "dSS.devices", "dSS.config");

dSS.devices.DeviceSettingsVideoView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.dSUID = config.dSUID;
        this.deviceRecord = config.device;
        this.apartmentMode = config.apartmentMode;

        Ext.apply(this, {
            padding: 5,
            layout: 'table',
            layoutConfig: {
                columns: 3,
                tableAttrs: {
                    width: '100%'
                }
            },
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme' ],
            items:
            [
                { xtype: 'box', height: 10, colspan: 3 },
                {
                    xtype: 'label',
                    text: dSS._("Power") + ':',
                    ctCls: 'x-text-align-right x-text-nowrap'
                },
                { xtype: 'box', width: 10 },
                {
                    xtype: 'panel',
                    ref: 'buttonPanel',
                    height: 22,
                    width: 200,
                    padding: 10,
                    layout: 'absolute',
                    border: false,
                    items:
                    [
                        {
                            xtype: 'button',
                            enableToggle: true,
                            toggleGroup: 'power',
                            ref: 'power_off',
                            minWidth: 45,
                            x: 0,
                            y: 0,
                            height: 12,
                            toggleHandler: this.powerToggled.createDelegate(this),
                            text: dSS._("off")
                        },
                        {
                            xtype: 'button',
                            toggleGroup: 'power',
                            enableToggle: true,
                            ref: 'power_on',
                            minWidth: 45,
                            x: 50,
                            y: 0,
                            height: 12,
                            toggleHandler: this.powerToggled.createDelegate(this),
                            text: dSS._("on")
                        }
                    ]
                },
                { xtype: 'box', height: 10, colspan: 3 },
                {
                    xtype: 'label',
                    text: dSS._("Signal input source") + ':',
                    ctCls: 'x-text-align-right x-text-nowrap'
                },
                { xtype: 'box', width: 10 },
                {
                    xtype: 'combo',
                    disabled: true,
                    width: 200,
                    ref: 'source',
                    lazyRender: true,
                    triggerAction: 'all',
                    forceSelection: true,
                    editable: false,
                    mode: 'local',
                    displayField: 'name',
                    valueField: 'id',
                    store: new Ext.data.Store({
                        root: 'root',
                        reader: new Ext.data.JsonReader({ root: 'root' },
                        Ext.data.Record.create([
                            { name: 'name' }, { name: 'id' }
                        ]))
                    })
                },
                { xtype: 'box', height: 10, colspan: 3 },
                {
                    xtype: 'label',
                    text: dSS._("Channel") + ':',
                    ctCls: 'x-text-align-right x-text-nowrap'
                },
                { xtype: 'box', width: 10 },
                {
                    xtype: 'numberfield',
                    disabled: true,
                    bubbleEvents: [ 'change' ],
                    ref: 'channel',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    width: 45,
                    listeners: {
                        change: this.handleChannelChanged.createDelegate(this)
                    }
                },
                { xtype: 'box', height: 10, colspan: 3 },
                {
                    xtype: 'label',
                    text: dSS._("Volume") + ':',
                    ctCls: 'x-text-align-right x-text-nowrap'
                },
                { xtype: 'box', width: 10 },
                {
                    xtype: 'panel',
                    ref: 'volumePanel',
                    height: 22,
                    width: 220,
                    padding: 10,
                    layout: 'absolute',
                    border: false,
                    items:
                    [
                        {
                            xtype: 'numberfield',
                            disabled: true,
                            bubbleEvents: [ 'change' ],
                            ref: 'volume',
                            allowBlank: false,
                            allowDecimals: true,
                            allowNegative: false,
                            decimalPrecision: 1,
                            width: 45,
                            minValue: 0,
                            maxValue: 100,
                            x: 0,
                            y: 0,
                            listeners: {
                                change: this.handleVolumeChanged.createDelegate(this)
                            }
                        },
                        {
                            xtype: 'label',
                            text: dSS._("%"),
                            x: 47,
                            y: 4
                        },
                        {
                            xtype: 'slider',
                            disabled: true,
                            ref: 'volumeSlider',
                            x: 65,
                            y: 1,
                            width: 135,
                            increment: 1,
                            minValue: 0,
                            maxValue: 100,
                            vertical: false,
                            decimalPrecision: 1,
                            plugins: new Ext.slider.Tip({
                                getText: function(thumb) {
                                    return String.format('{0}%', thumb.value);
                                }
                            }),
                            listeners: {
                                changeComplete: this.handleVolumeChanged.createDelegate(this)
                            }
                        }
                    ]
                }

            ]
        });

        dSS.devices.DeviceSettingsVideoView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.volumePanel.volume.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseInt(field.getValue());
                var ov = parseInt(this.volumePanel.volumeSlider.getValue());
                if (nv !== ov) {
                    this.handleVolumeChanged(field, nv);
                }
            }
        },
        this);

        this.channel.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseInt(field.getValue());
                this.handleChannelChanged(field, nv);
            }
        },
        this);

        this.source.on('change',
                this.handleSourceChanged.createDelegate(this), this);
    },

    handleSourceChanged: function(container, newVal) {
        if (typeof container.isValid === 'function') {
            if (!container.isValid()) {
                return;
            }
        }

        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }

        this.killRequests();

        if (this.apartmentMode !== true) {
            var params = {};
            params.channelvalues = 'videoInputSource=' + newVal;
            this.previousSource = newVal;
            this.setOutputChannel(params);
        }
    },

    handleVolumeChanged: function(container, newVal) {
        if (typeof container.isValid === 'function') {
            if (!container.isValid()) {
                return;
            }
        }

        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.volumePanel.volumeSlider.setValue(newVal);
        this.volumePanel.volume.setValue(newVal);
        if (this.apartmentMode !== true) {
            this.setVolumeOnDevice(newVal);
        }
    },

    setVolumeOnDevice: function(value) {
        var params = {};
        params.channelvalues = 'audioVolume=' + value;
        this.previousVolume = value;
        this.setOutputChannel(params);
    },

    handleChannelChanged: function(container, newVal) {
        if (!container.isValid()) {
            return;
        }

        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        if (this.apartmentMode !== true) {
            this.setChannelOnDevice(newVal);
        }
    },

    setChannelOnDevice: function(value) {
        var params = {};
        params.channelvalues = 'videoStation=' + value;
        this.setOutputChannel(params);
        this.previousChannel = value;
    },

    killRequests: function() {
        if (this.AjaxRequestVideoProperties !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestVideoProperties);
            this.AjaxRequestVideoProperties = undefined;
        }
    },
    enableElements: function(enable) {
        this.source.setDisabled(enable !== true);
        this.channel.setDisabled(enable !== true);
        this.volumePanel.volume.setDisabled(enable !== true);
        this.volumePanel.volumeSlider.setDisabled(enable  !== true);

    },
    powerToggled: function(button, state) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();

        if (state) {
            var params = {};
            var prev = 0;
            if (button.ref === "power_on") {
                params.channelvalues = 'powerState=1';
                prev = 1;
                this.enableElements(true);
            } else {
                params.channelvalues = 'powerState=0';
                this.enableElements(false);
            }

            if (this.apartmentMode !== true) {
                this.previousPower = prev;
                this.setOutputChannel(params);
            }
        }
    },
    setOutputChannel: function(params) {
        params.dsuid = this.dSUID;
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params: params,
            failureMsg: dSS._("Could not set device value.")
        });

    },
    setupVolume: function(state) {
        if ((state.min !== undefined) && (state.max !== undefined)) {
            this.volumePanel.volume.setMinValue(state.min);
            this.volumePanel.volume.setMaxValue(state.max);
            this.volumePanel.volumeSlider.setMinValue(state.min);
            this.volumePanel.volumeSlider.setMaxValue(state.max);
            this.volumePanel.volume.enable();
            this.volumePanel.volumeSlider.enable();
        }
    },
    setupVideoInput: function(state) {
        if (state.options !== undefined) {
            // of course the json coming from getInfo is in a form that
            // can't be parsed automatically.... doh
            var elements = [];
            var keys = Object.keys(state.options);
            var i;
            for (i = 0; i < keys.length; i++) {
                var el = {};
                el.id = parseInt(keys[i], 10);
                el.name = state.options[keys[i]];
                elements.push(el);
            }
            this.source.enable();
            this.source.getStore().loadData({ root: elements }, true);
        }
    },
    setupStation: function(state) {
        if ((state.min !== undefined) && (state.max !== undefined)) {
            this.channel.setMinValue(state.min);
            this.channel.setMaxValue(state.max);
            this.channel.enable();
        }
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }

        var powerState = '0';
        if (this.buttonPanel.power_on.pressed === true) {
            powerState = '1';
            this.enableElements(true);
        } else {
            this.enableElements(false);
        }

        var source = this.source.getValue();
        var channel = this.channel.getValue();
        var volume = this.volumePanel.volume.getValue();

        var params = {};
        params.channelvalues = 'videoStation=' + channel + ';' +
                               'powerState=' + powerState + ';' +
                               'audioVolume=' + volume + ';' +
                               'videoInputSource=' + source;

        this.setOutputChannel(params);
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }

        var params = {};
        params.channelvalues = '';

        if (this.previousPower != undefined) {
            params.channelvalues = params.channelvalues + 'powerState=' +
                this.previousPower + ';';
        }
        if (this.previousVolume != undefined) {
            params.channelvalues = params.channelvalues + 'audioVolume=' +
                this.previousVolume + ';';
            this.handleVolumeChanged(this.volumePanel.volume, this.previousVolume);
        }
        if (this.previousChannel != undefined) {
            params.channelvalues = params.channelvalues + 'videoStation=' +
                this.previousChannel + ';';
        }
        if (this.previousSource != undefined) {
            params.channelvalues = params.channelvalues + 'videoInputSource=' +
                this.previousSource + ';';
        }

        if (params.channelvalues.length > 0) {
            if (params.channelvalues[params.channelvalues.length - 1] === ';') {
                params.channelvalues = params.channelvalues.slice(0, -1);
            }
            this.setOutputChannel(params);
        }
        this.loadData(this.dSUID, this.deviceRecord);
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        if (this.apartmentMode !== true) {
            return;
        }

        this.killRequests();
       
        var powerState = '0';
        if (this.buttonPanel.power_on.pressed === true) {
            powerState = '1';
        }

        var source = this.source.getValue();
        var channel = this.channel.getValue();
        var volume = this.volumePanel.volume.getValue();

        var channelvalues = 'videoStation=' + channel + ';' +
                               'powerState=' + powerState + ';' +
                               'audioVolume=' + volume + ';' +
                               'videoInputSource=' + source;

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue',
            scope: this,
            params:
            {
                channelvalues: channelvalues,
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set device value.")
        });
        this.resetDefaults();
    },

    resetDefaults: function() {
        var powerState = '0';
        if (this.buttonPanel.power_on.pressed === true) {
            powerState = '1';
        }
        this.previousPower = powerState;
        this.previousSource = this.source.getValue();
        this.previousChannel = this.channel.getValue();
        this.previousVolume = this.volumePanel.volume.getValue();
    },
    loadData: function(reqdSUID, record) {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.deviceRecord = record;

        this.fireEvent('loading', true);

        this.AjaxRequestVideoProperties = Ext.Ajax.request({
            url: '/json/device/getInfo',
            disableCaching: true,
            method: 'GET',
            params:
            {
                dsuid: this.dSUID,
                lang: dSS.config.language
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestVideoProperties = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if ((jsonData.ok !== true) ||
                        (jsonData.result == undefined) ||
                        (jsonData.result.operational == undefined) ||
                        (jsonData.result.operational.outputs == undefined) ||
                        (jsonData.result.outputDescriptions == undefined)) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device information could not be loaded."));
                        return;
                    }

                    var outDesc = jsonData.result.outputDescriptions;

                    if (outDesc.audioVolume !== undefined) {
                        this.setupVolume(outDesc.audioVolume);
                    }

                    if (outDesc.videoInputSource !== undefined) {
                        this.setupVideoInput(outDesc.videoInputSource);
                    }

                    if (outDesc.videoStation !== undefined) {
                        this.setupStation(outDesc.videoStation);
                    }

                    var outputs = jsonData.result.operational.outputs;
                    if ((outputs.audioVolume !== undefined) &&
                        (outputs.audioVolume.value !== undefined)) {
                        this.volumePanel.volume.setValue(outputs.audioVolume.value);
                        this.volumePanel.volumeSlider.setValue(outputs.audioVolume.value);
                        this.previousVolume = outputs.audioVolume.value;
                    } else {
                        this.volumePanel.volume.markInvalid(dSS._("Device information could not be loaded."));
                    }

                    if ((outputs.videoStation !== undefined) &&
                        (outputs.videoStation.value !== undefined)) {
                        this.channel.setValue(outputs.videoStation.value);
                        this.previousChannel = outputs.videoStation.value;
                    } else {
                        this.channel.markInvalid(dSS._("Device information could not be loaded."));
                    }
                    if ((outputs.videoInputSource !== undefined) &&
                        (outputs.videoInputSource.value !== undefined)) {
                        this.source.setValue(outputs.videoInputSource.value);
                        this.previousSource = outputs.videoInputSource.value;
                    } else {
                        this.source.markInvalid(dSS._("Device information could not be loaded."));
                    }

                    if ((outputs.powerState !== undefined) &&
                        (outputs.powerState.value !== undefined)) {
                        if (outputs.powerState.value == 1) {
                            this.buttonPanel.power_on.toggle(true, true);
                            this.buttonPanel.power_off.toggle(false, true);
                            this.previousPower = 1;

                            this.enableElements(true);
                        } else {
                            this.buttonPanel.power_off.toggle(true, true);
                            this.buttonPanel.power_on.toggle(false, true);
                            this.previousPower = 0;
                            this.enableElements(false);
                        }
                    }
                    if (this.apartmentMode !== true) {
                        this.fireEvent('enableme');
                    }

                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Device information could not be loaded."));
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestVideoProperties = undefined;
                if (this.apartmentMode !== true) {
                    this.fireEvent('enableme');
                }

                if (this.closed === true) {
                    return;
                }
            }
        });
    }
});