//= require <dSS/devices/device-settings/device-type-panels/color-view/Components>
//= require <dSS/devices/device-settings/device-type-panels/color-view/Helpers>

Ext.namespace("dSS", "dSS.devices");

// *COLOR in apartment mode
dSS.devices.DeviceSettingsColorViewApartmentMode = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.previousColor = { h: -1, s: -1, v: -1 };
        this.defaultColor = { h: 0, s: 0, v: 0 };

        this.dSUID = config.dSUID;

        var panelItems = [
            {
                xtype: 'jpickerpanel',
                ref: 'colorPicker',
                border: false,
                localization: dSS.devices.helpers.localizePicker,
                hidden: false
            },
            new dSS.devices.components.SwitchToTemperatureModeButton({
                listeners: {
                    render: function (me) {
                        me.getEl().on('click', config.toggleView, me);
                    } 
                },
                hidden: true
            })

        ];
        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            border: false,
            bubbleEvents: [ 'formchanged', 'enableme' ],
            items: panelItems
        });

        dSS.devices.DeviceSettingsColorViewApartmentMode.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.closed = true;
        }, this);

        this.colorPicker.on('jpickercolorchanged', this.handleColorChanged.createDelegate(this), this);

    },
    setColorOnDevice: function(color) {
        // hue usually goes from 0 to 360°
        if (color.h > 360) {
            color.h = 360;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'hue=' + color.h +
                               ';saturation=' + color.s +
                               ';brightness=' + color.v,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set device color")
        });
    },
    handleColorChanged: function(panel, color) {
        this.fireEvent('formchanged');  
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }
        var color = this.colorPicker.getColor();
        this.setColorOnDevice(color);
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        var color = this.colorPicker.getColor();
        if (color.h > 360) {
            color.h = 360;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue2',
            scope: this,
            params:
            {
                channels: JSON.stringify({
                    hue: { value: color.h, dontCare: false },
                    saturation: { value: color.s, dontCare: false },
                    brightness: { value: color.v, dontCare: false }
                }),
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set device color")
        });
        this.resetDefaults();
    },
    clearData: function() {
        this.colorPicker.setColor(this.defaultColor);
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }
        this.previousColor = this.colorPicker.getColor();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        this.colorPicker.setColor(this.previousColor);
    },
    loadData: function(reqdSUID, deviceChannels) {
        this.dSUID = reqdSUID;
        this.fireEvent('enableme');

        if (deviceChannels.find(function (channel) {
            return channel.channelId === 'colortemp';
        })) {
            this.switchToTheOtherColorPicker.show();
        } else {
            this.switchToTheOtherColorPicker.hide();
        }
    },
    setColor: function(color) {
        this.previousColor = color;
        this.colorPicker.setColor(color);
    }
});