Ext.namespace("dSS", "dSS.devices", "dSS.devices.helpers");

if (!Object.freeze) {   // polyfill for older browsers
    Object.freeze = Object.getOwnPropertyNames && Object.preventExtensions && Object.defineProperty && Object.getOwnPropertyDescriptor
        ? function (obj) {
            var props = Object.getOwnPropertyNames(obj);
        
            for ( var i = 0; i < props.length; i++ ) {
                var desc = Object.getOwnPropertyDescriptor(obj, props[i]);
                if ('value' in desc) {
                    desc.writable = false;
                }
                desc.configurable = false;
                Object.defineProperty(obj, props[i], desc);
            }
            return Object.preventExtensions(obj);
        }
        : function () {};
}

dSS.devices.helpers.channelValueToTemperature = function (channelValue) {
    return Math.round(1000000 / channelValue);  // mired -> Kelvin
};
dSS.devices.helpers.temperatureToChannelValue = function (temperature) {
    return Math.round(1000000 / temperature);   // Kelvin -> mired
};

dSS.devices.helpers.colorTemperatureDefaults = Object.freeze({
    min: dSS.devices.helpers.channelValueToTemperature(500),    // in Kelvin
    max: dSS.devices.helpers.channelValueToTemperature(152.9),
    philipsHueWhiteAmbiance: Object.freeze({  // values for Philips Hue - hardcoded as there's no way to retrieve actual ones
        min: dSS.devices.helpers.channelValueToTemperature(453),  // should be 2000 in Kelvin, but for some reason isn't
        max: dSS.devices.helpers.channelValueToTemperature(152.9)  // should be 6500 K, but isn't :)
    }),
    philipsHueWhiteAndColorAmbiance: Object.freeze({
        min: dSS.devices.helpers.channelValueToTemperature(453),   // should be 2200 in Kelvin
        max: dSS.devices.helpers.channelValueToTemperature(152.9)    // should be 6600 K
    })
});

dSS.devices.helpers.localizePicker = {
    text: {
        title: dSS._("Drag markers to pick a color")
    },
    tooltips: {
        hue: {
            radio: dSS._("Set to Hue color mode"),
            textbox: dSS._("Enter a Hue value (0-360°)")
        },
        saturation: {
            radio: dSS._("Set to Saturation color mode"),
            textbox: dSS._("Enter a Saturation value (0-100%)")
        },
        value: {
            radio: dSS._("Set to Value color mode"),
            textbox: dSS._("Enter a Value value (0-100%)")
        }
    }
};

dSS.devices.helpers.ensureValueWithinRange = function (values) {
    if (values.max != null && values.min != null) {
        if (values.max < values.min) {
            throw new Error('Incorrect value range - min is greater than max.');
        }
        return Math.min(Math.max(values.value, values.min), values.max);
    }
    if (values.max != null) {
        return Math.min(values.value, values.max);
    }
    return Math.max(values.value, values.min);
}